@@ -14,7 +14,7 @@ my $topdir = File::Spec->rel2abs(dirname($0). "/..");
 my $lupdate_cmd = "lupdate";
 
 # Read all source files given in the parameter list and copy them to the
-# current directory, replaceing i18n by tr and I18N_NOOP by QT_TR_NOOP.
+# current directory, replaceing i18n by tr and I18N_NOOP by QT_TRANSLATE_NOOP.
 sub createTranslateSources(@)
 {
 	my @sources = @_;
@@ -24,7 +24,7 @@ sub createTranslateSources(@)
 		open OF, ">$outfn" or die "Could not create $outfn: $!\n";
 		while (<IF>) {
 			s/i18n\(/tr(/;
-			s/I18N_NOOP\(/QT_TR_NOOP(/;
+			s/I18N_NOOP\(/QT_TRANSLATE_NOOP("\@default", /;
 			print OF $_;
 		}				
 		close OF;
@@ -126,7 +126,7 @@ sub setTsTranslations($$%)
 # Generate .ts files from .po files.
 sub generateTs()
 {
-	my @languages = ("de", "es", "fr", "ru");
+	my @languages = ("de", "es", "fr", "ru", "it");
 	my $tmpdir = ".tsdir";
 	mkdir $tmpdir unless -d $tmpdir;
 	mkdir "po" unless -d "po";
@@ -149,6 +149,7 @@ my $have_vorbis = 1;
 my $have_flac = 1;
 my $have_id3lib = 1;
 my $have_taglib = 1;
+my $have_mp4v2 = 1;
 my $have_tunepimp = 5;
 my $qmake_cmd = "qmake";
 my $enable_pch = 0;
@@ -170,6 +171,8 @@ while (my $opt = shift) {
 		$have_id3lib = 0;
   } elsif ($opt eq "--without-taglib") {
 		$have_taglib = 0;
+  } elsif ($opt eq "--without-mp4v2") {
+		$have_mp4v2 = 0;
 	} elsif ($opt eq "--without-musicbrainz") {
 		$have_tunepimp = 0;
 	} elsif (substr($opt, 0, 19) eq "--with-musicbrainz=") {
@@ -204,6 +207,7 @@ while (my $opt = shift) {
 		print "  --without-musicbrainz  build without MusicBrainz\n";
 		print "  --with-musicbrainz=VER build with MusicBrainz version [5]\n";
 		print "  --without-taglib       build without taglib\n";
+		print "  --without-mp4v2        build without mp4v2\n";
 		print "  --without-id3lib       build without id3lib\n";
 		print "  --without-vorbis       build without ogg/vorbis\n";
 		print "  --without-flac         build without FLAC\n";
@@ -244,6 +248,7 @@ if ($from_configure) {
 	$have_vorbis = 0;
 	$have_flac = 0;
 	$have_taglib = 0;
+	$have_mp4v2 = 0;
 	$have_tunepimp = 0;
 	$qmake_cmd = "";
 	$lupdate_cmd = "";
@@ -272,6 +277,8 @@ if ($from_configure) {
 				$have_flac = $1;
 			} elsif (/^#define HAVE_TAGLIB (\d+)$/) {
 				$have_taglib = $1;
+			} elsif (/^#define HAVE_MP4V2 (\d+)$/) {
+				$have_mp4v2 = $1;
 			} elsif (/^#define HAVE_TUNEPIMP (\d+)$/) {
 				$have_tunepimp = $1;
 			} elsif (/^#define CFG_QMAKE "(.*)"$/) {
@@ -288,7 +295,7 @@ if ($from_configure) {
 	}
 }
 
-my $config_h = "#define VERSION \"0.9\"\n";
+my $config_h = "#define VERSION \"0.10\"\n";
 my $config_pri;
 if ($prefix) {
 	$config_h .= "#define CFG_PREFIX \"$prefix\"\n";
@@ -347,6 +354,10 @@ if ($have_taglib) {
 	$config_h .= "#define HAVE_TAGLIB $have_taglib\n";
 	$config_pri .= "-ltag ";
 }
+if ($have_mp4v2) {
+	$config_h .= "#define HAVE_MP4V2 $have_mp4v2\n";
+	$config_pri .= "-lmp4v2 ";
+}
 if ($have_tunepimp) {
 	$config_h .= "#define HAVE_TUNEPIMP $have_tunepimp\n";
 	$config_pri .= "-ltunepimp ";
@@ -470,6 +481,7 @@ if (open IF, "$topdir/kid3/kid3.desktop") {
 	while (<IF>) {
 		s/^Name=Kid3$/Name=Kid3-qt/;
 		s/^Exec=kid3.*$/Exec=kid3-qt/;
+		s/^Icon=kid3$/Icon=kid3-qt/;
 		print OF $_;
 	}
 	print "creating $fn\n";
