@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 17 Sep 2003
+ *
+ * Copyright (C) 2003-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef CONFIGDIALOG_H
@@ -22,11 +39,7 @@
 #ifdef KID3_USE_KCONFIGDIALOG
 #include <kconfigdialog.h>
 #else
-#if QT_VERSION >= 0x040000
-#include <Q3TabDialog>
-#else
-#include <qtabdialog.h>
-#endif
+#include <qdialog.h>
 #endif
 #include "formatconfig.h"
 
@@ -45,7 +58,7 @@ class QLineEdit;
 #ifdef KID3_USE_KCONFIGDIALOG
 typedef KConfigDialog ConfigDialogBaseClass;
 #else
-typedef Q3TabDialog ConfigDialogBaseClass;
+typedef QDialog ConfigDialogBaseClass;
 #endif
 
 /**
@@ -96,6 +109,23 @@ protected slots:
 	 */
 	virtual void slotHelp();
 
+	/**
+	 * Select custom application font.
+	 */
+	void slotSelectFont();
+
+	/**
+	 * Select custom application style.
+	 *
+	 * @param key style key
+	 */
+	void slotSelectStyle(const QString& key);
+
+	/**
+	 * Revert the font and style to the values in the settings.
+	 */
+	void slotRevertFontAndStyle();
+
 private:
 	/** Preserve timestamp checkbox */
 	QCheckBox* m_preserveTimeCheckBox;
@@ -105,8 +135,12 @@ private:
 	QCheckBox* m_totalNumTracksCheckBox;
 	/** Comment field name combo box */
 	QComboBox* m_commentNameComboBox;
+	/** ID3v2 text encoding combo box */
+	QComboBox* m_textEncodingComboBox;
+#if defined HAVE_ID3LIB && defined HAVE_TAGLIB
 	/** ID3v2 version combo box */
 	QComboBox* m_id3v2VersionComboBox;
+#endif
 	/** Filename Format box */
 	FormatBox* m_fnFormatBox;
 	/** ID3 Format box */
@@ -123,6 +157,16 @@ private:
 	QCheckBox* m_proxyCheckBox;
 	/** Proxy line edit */
 	QLineEdit* m_proxyLineEdit;
+#ifndef CONFIG_USE_KDE
+	QCheckBox* m_useApplicationFontCheckBox;
+	QPushButton* m_applicationFontButton;
+	QCheckBox* m_useApplicationStyleCheckBox;
+	QComboBox* m_applicationStyleComboBox;
+	QFont m_font;
+	QString m_style;
+	bool m_fontChanged;
+	bool m_styleChanged;
+#endif
 };
 
 #endif
