@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 9 Jan 2003
+ *
+ * Copyright (C) 2003-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef FILELIST_H
@@ -14,9 +31,12 @@
 #include <qsize.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
-#include <Q3ListView>
+#include <QTreeWidget>
+typedef QTreeWidget FileListBaseClass;
 #else
 #include <qlistview.h>
+typedef QListView FileListBaseClass;
+class QAction;
 #endif
 #include "dirinfo.h"
 
@@ -27,15 +47,16 @@ class ExternalProcess;
 /**
  * List of files to operate on.
  */
-class FileList : public Q3ListView
+class FileList : public FileListBaseClass
 {
 Q_OBJECT
 
 public:
 	/**
 	 * Constructor.
+	 * @param parent parent widget
 	 */
-	FileList(QWidget* parent = 0, const char* name = 0, Qt::WFlags f = 0);
+	FileList(QWidget* parent = 0);
 
 	/**
 	 * Destructor.
@@ -96,10 +117,12 @@ public:
 	/**
 	 * Fill the filelist with the files found in a directory.
 	 *
-	 * @param name path of directory
+	 * @param name     path of directory
+	 * @param fileName name of file to select (optional, else empty)
+	 *
 	 * @return false if name is not directory path, else true.
 	 */
-	bool readDir(const QString& name);
+	bool readDir(const QString& name, const QString& fileName = QString());
 
 	/**
 	 * Refresh text of all files in listview and check if any file is modified.
@@ -108,25 +131,17 @@ public:
 	 */
 	bool updateModificationState();
 
-	/**
-	 * Create a TaggedFile subclass depending on the file extension.
-	 *
-	 * @param di directory information
-	 * @param fn filename
-	 *
-	 * @return tagged file, 0 if no type found.
-	 */
-	static TaggedFile* createTaggedFile(const DirInfo* di, const QString& fn);
-
 	/**
 	 * Fill the filelist with the files found in the directory tree.
 	 *
 	 * @param dirInfo  information  about directory
 	 * @param item     parent directory item or 0 if top-level
 	 * @param listView parent list view if top-level, else 0
+	 * @param fileName name of file to select (optional, else empty)
 	 */
 	static void readSubDirectory(DirInfo* dirInfo, FileListItem* item,
-															 FileList* listView);
+															 FileList* listView,
+															 const QString& fileName = QString());
 
 signals:
 	/**
@@ -142,7 +157,7 @@ private slots:
 	 * @param pos  position where context menu is drawn on screen
 	 */
 #if QT_VERSION >= 0x040000
-	void contextMenu(Q3ListViewItem* item, const QPoint& pos);
+	void contextMenu(QTreeWidgetItem* item, const QPoint& pos);
 #else
 	void contextMenu(QListViewItem* item, const QPoint& pos);
 #endif
@@ -154,6 +169,13 @@ private slots:
 	 */
 	void executeContextCommand(int id);
 
+	/**
+	 * Execute a context menu action.
+	 *
+	 * @param action action of selected menu
+	 */
+	void executeAction(QAction* action);
+
 	/**
 	 * Rename the selected file(s).
 	 */
@@ -164,6 +186,34 @@ private slots:
 	 */
 	void deleteFile();
 
+	/**
+	 * Expand an item.
+	 *
+	 * @param item item
+	 */
+	void expandItem(QTreeWidgetItem* item);
+
+	/**
+	 * Collapse an item.
+	 *
+	 * @param item item
+	 */
+	void collapseItem(QTreeWidgetItem* item);
+
+	/**
+	 * Display a custom context menu with operations for selected files.
+	 *
+	 * @param pos  position where context menu is drawn on screen
+	 */
+	void customContextMenu(const QPoint& pos);
+
+	/**
+	 * Expand or collapse an item which has no children.
+	 *
+	 * @param item item
+	 */
+	void expandOrCollapseEmptyItem(QTreeWidgetItem* item);
+
 private:
 	FileList(const FileList&);
 	FileList& operator=(const FileList&);
@@ -194,7 +244,11 @@ private:
 	/** information about directory */
 	DirInfo m_dirInfo;
 	/** iterator pointing to current file */
-	Q3ListViewItemIterator m_iterator;
+#if QT_VERSION >= 0x040000
+	QTreeWidgetItemIterator* m_iterator;
+#else
+	QListViewItemIterator m_iterator;
+#endif
 	/** current item in current directory */
 	FileListItem* m_currentItemInDir;
 	/** Process for context menu commands */
