@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 17 Sep 2003
+ *
+ * Copyright (C) 2003-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef FORMATBOX_H
@@ -12,11 +29,12 @@
 
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
-#include <Q3GroupBox>
-class Q3Table;
+#include <QGroupBox>
+class QTableWidget;
 #else
 #include <qgroupbox.h>
 class QTable;
+class QAction;
 #endif
 
 class QComboBox;
@@ -28,7 +46,7 @@ class FormatConfig;
 /**
  * Group box containing format options.
  */
-class FormatBox : public Q3GroupBox
+class FormatBox : public QGroupBox
 {
 Q_OBJECT
 
@@ -38,9 +56,8 @@ public:
 	 *
 	 * @param title  title
 	 * @param parent parent widget
-	 * @param name   Qt object name
 	 */
-	FormatBox(const QString& title, QWidget* parent = 0, const char* name = 0);
+	FormatBox(const QString& title, QWidget* parent = 0);
 
 	/**
 	 * Destructor.
@@ -94,6 +111,13 @@ public slots:
 	 */
 	void clearCell(int row_col);
 
+	/**
+	 * Execute a context menu action.
+	 *
+	 * @param action action of selected menu
+	 */
+	void executeAction(QAction* action);
+
 	/**
 	 * Display context menu.
 	 *
@@ -103,10 +127,21 @@ public slots:
 	 */
 	void contextMenu(int row, int col, const QPoint& pos);
 
+	/**
+	 * Display custom context menu.
+	 *
+	 * @param pos position where context menu is drawn on screen
+	 */
+	void customContextMenu(const QPoint& pos);
+
 private:
 	QComboBox* m_caseConvComboBox;
 	QCheckBox* m_strRepCheckBox;
-	Q3Table* m_strReplTable;
+#if QT_VERSION >= 0x040000
+	QTableWidget* m_strReplTable;
+#else
+	QTable* m_strReplTable;
+#endif
 	QCheckBox* m_formatEditingCheckBox;
 };
 
