@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 17 Sep 2003
+ *
+ * Copyright (C) 2003-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef IMPORTCONFIG_H
@@ -19,6 +36,15 @@
  */
 class ImportConfig : public GeneralConfig {
 public:
+	/** Import destinations */
+	enum ImportDestination { DestV1, DestV2, DestV1V2 };
+
+	/** Import servers */
+	enum ImportServer {
+		ServerFreedb, ServerTrackType, ServerDiscogs,
+		ServerMusicBrainzRelease, ServerMusicBrainzFingerprint
+	};
+
 	/**
 	 * Constructor.
 	 * Set default configuration.
@@ -58,8 +84,10 @@ public:
 #endif
 		);
 
+	/** import server */
+	ImportServer m_importServer;
 	/** true to import into ID3v1 tags, else into ID3v2 tags */
-	bool m_importDestV1;
+	ImportDestination m_importDest;
 	/** Names of import formats */
 	QStringList m_importFormatNames;
 	/** regexp describing header import format */
