@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 17 Sep 2003
+ *
+ * Copyright (C) 2003-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -38,11 +55,12 @@
  */
 ImportDialog::ImportDialog(QWidget* parent, QString& caption,
 													 ImportTrackDataVector& trackDataList) :
-	QDialog(parent, "import", true),
+	QDialog(parent),
 	m_autoStartSubDialog(ASD_None),
 	m_trackDataVector(trackDataList)
 {
-	setCaption(caption);
+	setModal(true);
+	QCM_setWindowTitle(caption);
 
 	QVBoxLayout* vlayout = new QVBoxLayout(this);
 	if (!vlayout) {
@@ -53,7 +71,7 @@ ImportDialog::ImportDialog(QWidget* parent, QString& caption,
 	m_impsel = new ImportSelector(this, m_trackDataVector);
 	vlayout->addWidget(m_impsel);
 
-	QHBoxLayout* hlayout = new QHBoxLayout(vlayout);
+	QHBoxLayout* hlayout = new QHBoxLayout;
 	QSpacerItem* hspacer = new QSpacerItem(16, 0, QSizePolicy::Expanding,
 	                                       QSizePolicy::Minimum);
 	QPushButton* helpButton = new QPushButton(i18n("&Help"), this);
@@ -71,6 +89,7 @@ ImportDialog::ImportDialog(QWidget* parent, QString& caption,
 		connect(saveButton, SIGNAL(clicked()), m_impsel, SLOT(saveConfig()));
 		connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
 		connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
+		vlayout->addLayout(hlayout);
 	}
 }
 
@@ -112,6 +131,39 @@ int ImportDialog::exec()
 	return QDialog::exec();
 }
 
+/**
+ * Set dialog to be started automatically.
+ *
+ * @param asd dialog to be started
+ */
+void ImportDialog::setAutoStartSubDialog(AutoStartSubDialog asd)
+{
+	m_autoStartSubDialog = asd;
+
+	ImportConfig::ImportServer server;
+	switch (asd) {
+		case ASD_Freedb:
+			server = ImportConfig::ServerFreedb;
+			break;
+		case ASD_TrackType:
+			server = ImportConfig::ServerTrackType;
+			break;
+		case ASD_Discogs:
+			server = ImportConfig::ServerDiscogs;
+			break;
+		case ASD_MusicBrainzRelease:
+			server = ImportConfig::ServerMusicBrainzRelease;
+			break;
+		case ASD_MusicBrainz:
+			server = ImportConfig::ServerMusicBrainzFingerprint;
+			break;
+		case ASD_None:
+		default:
+			return;
+	}
+	m_impsel->setImportServer(server);
+}
+
 /**
  * Clear dialog data.
  */
@@ -123,12 +175,11 @@ void ImportDialog::clear()
 /**
  * Get import destination.
  *
- * @return true if ID3v1 is destination,
- *         false if ID3v2.
+ * @return DestV1, DestV2 or DestV1V2 for ID3v1, ID3v2 or both.
  */
-bool ImportDialog::getDestV1() const
+ImportConfig::ImportDestination ImportDialog::getDestination() const
 {
-	return (m_impsel->getDestination() == ImportSelector::DestV1);
+	return m_impsel->getDestination();
 }
 
 /**
