@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 17 Sep 2003
+ *
+ * Copyright (C) 2003-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef IMPORTDIALOG_H
@@ -12,6 +29,7 @@
 
 #include "config.h"
 #include "importtrackdata.h"
+#include "importconfig.h"
 #include <qdialog.h>
 
 class ImportSelector;
@@ -61,9 +79,7 @@ public:
 	 *
 	 * @param asd dialog to be started
 	 */
-	void setAutoStartSubDialog(AutoStartSubDialog asd) {
-		m_autoStartSubDialog = asd;
-	}
+	void setAutoStartSubDialog(AutoStartSubDialog asd);
 
 	/**
 	 * Clear dialog data.
@@ -73,10 +89,9 @@ public:
 	/**
 	 * Get import destination.
 	 *
-	 * @return true if ID3v1 is destination,
-	 *         false if ID3v2.
+	 * @return DestV1, DestV2 or DestV1V2 for ID3v1, ID3v2 or both.
 	 */
-	bool getDestV1() const;
+	ImportConfig::ImportDestination getDestination() const;
 
 public slots:
 	/**
