@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 17 Sep 2003
+ *
+ * Copyright (C) 2003-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -27,18 +44,18 @@
 #include <qspinbox.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
-#include <Q3Table>
-#include <Q3ValueList>
+#include <QTableWidget>
+#include <QHeaderView>
+#include <QList>
 #include <QHBoxLayout>
 #include <QGridLayout>
-#include <Q3GroupBox>
+#include <QGroupBox>
 #else
 #include <qtable.h>
 #include <qgroupbox.h>
 #endif
 #include "genres.h"
 #include "standardtags.h"
-#include "importparser.h"
 #include "freedbdialog.h"
 #include "tracktypedialog.h"
 #include "musicbrainzreleasedialog.h"
@@ -55,30 +72,56 @@
  * Table used for import data.
  * Subclassed to be able to change the cell colors.
  */
-class ImportTable: public Q3Table {
+class ImportTable: public
+#if QT_VERSION >= 0x040000
+QTableWidget
+#else
+QTable
+#endif
+{
 public:
 	/**
 	 * Constructor.
 	 * @param parent parent widget
-	 * @param name   Qt name
 	 */
-	ImportTable(QWidget* parent = 0, const char* name = 0) :
-		Q3Table(parent, name) {}
+	ImportTable(QWidget* parent = 0) :
+#if QT_VERSION >= 0x040000
+		QTableWidget(parent)
+#else
+		QTable(parent)
+#endif
+		{}
 
 	/**
 	 * Constructor.
 	 * @param numRows number of rows
 	 * @param numCols number of columns
 	 * @param parent  parent widget
-	 * @param name    Qt name
 	 */
-	ImportTable(int numRows, int numCols, QWidget* parent = 0, const char* name = 0) :
-		Q3Table(numRows, numCols, parent, name) {} 
+	ImportTable(int numRows, int numCols, QWidget* parent = 0) :
+#if QT_VERSION >= 0x040000
+		QTableWidget(numRows, numCols, parent)
+#else
+		QTable(numRows, numCols, parent)
+#endif
+		{} 
 
 	/**
 	 * Clear marked rows.
 	 */
-	void clearMarks() { m_markedRows.fill(false); }
+	void clearMarks() {
+		m_markedRows.fill(false);
+#if QT_VERSION >= 0x040000
+		for (int row = 0; row < rowCount(); ++row) {
+			QTableWidgetItem* twi = item(row, 0);
+#if QT_VERSION >= 0x040200
+			if (twi) twi->setBackground(Qt::NoBrush);
+#else
+			if (twi) twi->setBackgroundColor(QColor());
+#endif
+		}
+#endif
+	}
 
 	/**
 	 * Mark a row.
@@ -86,8 +129,16 @@ public:
 	 * @param row number of row
 	 */
 	void markRow(unsigned row) {
-		if (m_markedRows.size() <= row) m_markedRows.resize(row + 1);
+		if (static_cast<unsigned>(m_markedRows.size()) <= row)
+			m_markedRows.resize(row + 1);
 		m_markedRows.setBit(row);
+#if QT_VERSION >= 0x040200
+		QTableWidgetItem* twi = item(row, 0);
+		if (twi) twi->setBackground(Qt::red);
+#elif QT_VERSION >= 0x040000
+		QTableWidgetItem* twi = item(row, 0);
+		if (twi) twi->setBackgroundColor(Qt::red);
+#endif
 	}
 
 	/**
@@ -96,7 +147,8 @@ public:
 	 * @return true if row is marked.
 	 */
 	bool isRowMarked(unsigned row) const {
-		return m_markedRows.size() > row && m_markedRows.testBit(row);
+		return static_cast<unsigned>(m_markedRows.size()) > row &&
+			m_markedRows.testBit(row);
 	}
 
 protected:
@@ -110,34 +162,36 @@ protected:
 	 * @param selected true if selected
 	 * @param cg color group
 	 */
+#if QT_VERSION < 0x040000
 	virtual void paintCell(QPainter* p, int row, int col, const QRect& cr, bool selected, const QColorGroup& cg) {
 		if (col == 0 && isRowMarked(row)) {
 			QColorGroup g(cg);
 			g.setColor(QColorGroup::Base, QColor("red"));
-			Q3Table::paintCell(p, row, col, cr, selected, g);
+			QTable::paintCell(p, row, col, cr, selected, g);
 		} else {
-			Q3Table::paintCell(p, row, col, cr, selected, cg);
+			QTable::paintCell(p, row, col, cr, selected, cg);
 		}
 	}
+#endif
 
 private:
 	QBitArray m_markedRows;
 };
 
 
+/** Last directory used for import or export. */
+QString ImportSelector::s_importDir;
+
 /**
  * Constructor.
  *
  * @param parent parent widget
  * @param trackDataList track data to be filled with imported values,
  *                      is passed with durations of files set
- * @param name          Qt name
- * @param f             window flags
  */
 ImportSelector::ImportSelector(
-	QWidget* parent, ImportTrackDataVector& trackDataList,
-	const char* name, Qt::WFlags f) :
-	Q3VBox(parent, name, f),
+	QWidget* parent, ImportTrackDataVector& trackDataList) :
+	QWidget(parent),
 	m_trackDataVector(trackDataList)
 {
 	m_freedbDialog = 0;
@@ -150,15 +204,37 @@ ImportSelector::ImportSelector(
 	m_importSource = None;
 	m_headerParser = new ImportParser();
 	m_trackParser = new ImportParser();
-	setSpacing(6);
-	setMargin(6);
+	QVBoxLayout* vboxLayout = new QVBoxLayout(this);
+	vboxLayout->setSpacing(6);
+	vboxLayout->setMargin(6);
+#if QT_VERSION >= 0x040000
+	m_tab = new ImportTable(0, NumColumns, this);
+	m_tab->setSelectionMode(QAbstractItemView::NoSelection);
+	m_tab->setHorizontalHeaderLabels(
+		QStringList() << i18n("Length") << i18n("Track") << i18n("Title") <<
+		i18n("Artist") << i18n("Album") << i18n("Year") << i18n("Genre") <<
+		i18n("Comment"));
+	m_tab->resizeColumnToContents(TrackColumn);
+	m_tab->resizeColumnToContents(YearColumn);
+	QHeaderView* vHeader = m_tab->verticalHeader();
+	if (vHeader) {
+		vHeader->setMovable(true);
+		vHeader->setResizeMode(
+#if QT_VERSION >= 0x040200
+			QHeaderView::ResizeToContents
+#else
+			QHeaderView::Stretch
+#endif
+			);
+	}
+#else
 	m_tab = new ImportTable(0, NumColumns, this);
 	m_tab->setReadOnly(true);
-	m_tab->setFocusStyle(Q3Table::FollowStyle);
+	m_tab->setFocusStyle(QTable::FollowStyle);
 	m_tab->setRowMovingEnabled(true);
-	m_tab->setSelectionMode(Q3Table::NoSelection);
-	Q3Header* hHeader = m_tab->horizontalHeader();
-	Q3Header* vHeader = m_tab->verticalHeader();
+	m_tab->setSelectionMode(QTable::NoSelection);
+	QHeader* hHeader = m_tab->horizontalHeader();
+	QHeader* vHeader = m_tab->verticalHeader();
 	hHeader->setLabel(LengthColumn, i18n("Length"));
 	hHeader->setLabel(TrackColumn, i18n("Track"));
 	hHeader->setLabel(TitleColumn, i18n("Title"));
@@ -169,82 +245,101 @@ ImportSelector::ImportSelector(
 	hHeader->setLabel(CommentColumn, i18n("Comment"));
 	m_tab->adjustColumn(TrackColumn);
 	m_tab->adjustColumn(YearColumn);
-
-	Q3GroupBox* fmtbox = new Q3GroupBox(3, Qt::Vertical, i18n("Format"), this);
-#if QT_VERSION >= 0x040000
-	fmtbox->setInsideMargin(5);
 #endif
-	m_formatComboBox = new QComboBox(false, fmtbox, "formatComboBox");
+	vboxLayout->addWidget(m_tab);
+
+#if QT_VERSION >= 0x040000
+	QGroupBox* fmtbox = new QGroupBox(i18n("Format"), this);
+#else
+	QGroupBox* fmtbox = new QGroupBox(3, Qt::Vertical, i18n("Format"), this);
+#endif
+	m_formatComboBox = new QComboBox(fmtbox);
 	m_formatComboBox->setEditable(true);
 	m_headerLineEdit = new QLineEdit(fmtbox);
 	m_trackLineEdit = new QLineEdit(fmtbox);
 	connect(m_formatComboBox, SIGNAL(activated(int)), this, SLOT(setFormatLineEdit(int)));
+#if QT_VERSION >= 0x040000
+	QVBoxLayout* vbox = new QVBoxLayout;
+	vbox->setMargin(2);
+	vbox->addWidget(m_formatComboBox);
+	vbox->addWidget(m_headerLineEdit);
+	vbox->addWidget(m_trackLineEdit);
+	fmtbox->setLayout(vbox);
+#endif
+	vboxLayout->addWidget(fmtbox);
 
 	QWidget* butbox = new QWidget(this);
-	QGridLayout* butlayout = new QGridLayout(butbox, 2, 8, 0, 6);
+	QHBoxLayout* butlayout = new QHBoxLayout(butbox);
+	butlayout->setMargin(0);
+	butlayout->setSpacing(6);
 	m_fileButton = new QPushButton(i18n("From F&ile"), butbox);
-	butlayout->addWidget(m_fileButton, 0, 0);
+	butlayout->addWidget(m_fileButton);
 	m_clipButton = new QPushButton(i18n("From Clip&board"), butbox);
-	butlayout->addWidget(m_clipButton, 0, 1);
-	m_freedbButton = new QPushButton(i18n("&gnudb.org"), butbox);
-	butlayout->addWidget(m_freedbButton, 0, 2);
-	m_trackTypeButton = new QPushButton(i18n("&TrackType.org"), butbox);
-	butlayout->addWidget(m_trackTypeButton, 0, 3);
-	m_discogsButton = new QPushButton(i18n("Disco&gs"), butbox);
-	butlayout->addWidget(m_discogsButton, 0, 4);
+	butlayout->addWidget(m_clipButton);
+	m_serverButton = new QPushButton(i18n("&From Server:"), butbox);
+	butlayout->addWidget(m_serverButton);
+	m_serverComboBox = new QComboBox(butbox);
+	m_serverComboBox->setEditable(false);
+	m_serverComboBox->QCM_insertItem(ImportConfig::ServerFreedb, i18n("gnudb.org"));
+	m_serverComboBox->QCM_insertItem(ImportConfig::ServerTrackType, i18n("TrackType.org"));
+	m_serverComboBox->QCM_insertItem(ImportConfig::ServerDiscogs, i18n("Discogs"));
+	m_serverComboBox->QCM_insertItem(ImportConfig::ServerMusicBrainzRelease,
+																	 i18n("MusicBrainz Release"));
+#ifdef HAVE_TUNEPIMP
+	m_serverComboBox->QCM_insertItem(ImportConfig::ServerMusicBrainzFingerprint,
+																	 i18n("MusicBrainz Fingerprint"));
+#endif
+	butlayout->addWidget(m_serverComboBox);
 	QSpacerItem* butspacer = new QSpacerItem(16, 0, QSizePolicy::Expanding,
 	                                       QSizePolicy::Minimum);
-	butlayout->addItem(butspacer, 0, 5);
-	QLabel* destLabel = new QLabel(butbox, "destLabel");
+	butlayout->addItem(butspacer);
+	QLabel* destLabel = new QLabel(butbox);
 	destLabel->setText(i18n("D&estination:"));
-	butlayout->addWidget(destLabel, 0, 6);
-	m_destComboBox = new QComboBox(false, butbox, "destComboBox");
-	m_destComboBox->insertItem(i18n("Tag 1"), DestV1);
-	m_destComboBox->insertItem(i18n("Tag 2"), DestV2);
+	butlayout->addWidget(destLabel);
+	m_destComboBox = new QComboBox(butbox);
+	m_destComboBox->setEditable(false);
+	m_destComboBox->QCM_insertItem(ImportConfig::DestV1, i18n("Tag 1"));
+	m_destComboBox->QCM_insertItem(ImportConfig::DestV2, i18n("Tag 2"));
+	m_destComboBox->QCM_insertItem(ImportConfig::DestV1V2, i18n("Tag 1 and Tag 2"));
 	destLabel->setBuddy(m_destComboBox);
-	butlayout->addWidget(m_destComboBox, 0, 7);
+	butlayout->addWidget(m_destComboBox);
+	vboxLayout->addWidget(butbox);
 
-	m_musicBrainzReleaseButton = new QPushButton(i18n("From MusicBrainz &Release"), butbox);
-	butlayout->addMultiCellWidget(m_musicBrainzReleaseButton, 1, 1, 0, 1);
-#ifdef HAVE_TUNEPIMP
-	m_musicBrainzButton = new QPushButton(i18n("From &MusicBrainz Fingerprint"), butbox);
-	butlayout->addMultiCellWidget(m_musicBrainzButton, 1, 1, 2, 4);
-#endif
-
-	QWidget* matchBox = new QWidget(this, "matchBox");
-	QHBoxLayout* matchLayout = new QHBoxLayout(matchBox, 0, 6, "matchLayout");
+	QWidget* matchBox = new QWidget(this);
+	QHBoxLayout* matchLayout = new QHBoxLayout(matchBox);
+	matchLayout->setMargin(0);
+	matchLayout->setSpacing(6);
 	m_mismatchCheckBox = new QCheckBox(
-		i18n("Check maximum allowable time &difference (sec):"), matchBox ,
-		"mismatchCheckBox");
+		i18n("Check maximum allowable time &difference (sec):"), matchBox);
 	matchLayout->addWidget(m_mismatchCheckBox);
-	m_maxDiffSpinBox = new QSpinBox(matchBox, "maxDiffSpinBox");
-	m_maxDiffSpinBox->setMaxValue(9999);
+	m_maxDiffSpinBox = new QSpinBox(matchBox);
+	m_maxDiffSpinBox->QCM_setMaximum(9999);
 	matchLayout->addWidget(m_maxDiffSpinBox);
 	QSpacerItem* matchSpacer = new QSpacerItem(16, 0, QSizePolicy::Expanding,
 																						 QSizePolicy::Minimum);
 	matchLayout->addItem(matchSpacer);
-	QLabel* matchLabel = new QLabel(i18n("Match with:"), matchBox, "matchLabel");
+	QLabel* matchLabel = new QLabel(i18n("Match with:"), matchBox);
 	matchLayout->addWidget(matchLabel);
-	m_lengthButton = new QPushButton(i18n("&Length"), matchBox, "lengthButton");
+	m_lengthButton = new QPushButton(i18n("&Length"), matchBox);
 	matchLayout->addWidget(m_lengthButton);
-	m_trackButton = new QPushButton(i18n("T&rack"), matchBox, "trackButton");
+	m_trackButton = new QPushButton(i18n("T&rack"), matchBox);
 	matchLayout->addWidget(m_trackButton);
-	m_titleButton = new QPushButton(i18n("&Title"), matchBox, "titleButton");
+	m_titleButton = new QPushButton(i18n("&Title"), matchBox);
 	matchLayout->addWidget(m_titleButton);
+	vboxLayout->addWidget(matchBox);
 
 	connect(m_fileButton, SIGNAL(clicked()), this, SLOT(fromFile()));
 	connect(m_clipButton, SIGNAL(clicked()), this, SLOT(fromClipboard()));
-	connect(m_freedbButton, SIGNAL(clicked()), this, SLOT(fromFreedb()));
-	connect(m_trackTypeButton, SIGNAL(clicked()), this, SLOT(fromTrackType()));
-	connect(m_musicBrainzReleaseButton, SIGNAL(clicked()), this, SLOT(fromMusicBrainzRelease()));
-	connect(m_discogsButton, SIGNAL(clicked()), this, SLOT(fromDiscogs()));
-#ifdef HAVE_TUNEPIMP
-	connect(m_musicBrainzButton, SIGNAL(clicked()), this, SLOT(fromMusicBrainz()));
-#endif
+	connect(m_serverButton, SIGNAL(clicked()), this, SLOT(fromServer()));
+	connect(m_serverComboBox, SIGNAL(activated(int)), this, SLOT(fromServer()));
 	connect(m_lengthButton, SIGNAL(clicked()), this, SLOT(matchWithLength()));
 	connect(m_trackButton, SIGNAL(clicked()), this, SLOT(matchWithTrack()));
 	connect(m_titleButton, SIGNAL(clicked()), this, SLOT(matchWithTitle()));
+#if QT_VERSION >= 0x040000
+	connect(vHeader, SIGNAL(sectionMoved(int, int, int)), this, SLOT(moveTableRow(int, int, int)));
+#else
 	connect(vHeader, SIGNAL(indexChange(int, int, int)), this, SLOT(moveTableRow(int, int, int)));
+#endif
 	connect(m_mismatchCheckBox, SIGNAL(toggled(bool)), this, SLOT(showPreview()));
 	connect(m_maxDiffSpinBox, SIGNAL(valueChanged(int)), this, SLOT(maxDiffChanged()));
 }
@@ -285,20 +380,34 @@ ImportSelector::~ImportSelector()
 #endif
 }
 
+/**
+ * Select the import server.
+ *
+ * @param server import server
+ */
+void ImportSelector::setImportServer(ImportConfig::ImportServer server)
+{
+	m_serverComboBox->QCM_setCurrentIndex(server);
+}
+
 /**
  * Clear dialog data.
  */
 void ImportSelector::clear()
 {
+#if QT_VERSION >= 0x040000
+	m_tab->setRowCount(0);
+#else
 	m_tab->setNumRows(0);
-	m_destComboBox->setCurrentItem(
-		static_cast<int>(Kid3App::s_genCfg.m_importDestV1 ? DestV1 : DestV2));
+#endif
+	m_serverComboBox->QCM_setCurrentIndex(Kid3App::s_genCfg.m_importServer);
+	m_destComboBox->QCM_setCurrentIndex(Kid3App::s_genCfg.m_importDest);
 
 	m_formatHeaders = Kid3App::s_genCfg.m_importFormatHeaders;
 	m_formatTracks = Kid3App::s_genCfg.m_importFormatTracks;
 	m_formatComboBox->clear();
-	m_formatComboBox->insertStringList(Kid3App::s_genCfg.m_importFormatNames);
-	m_formatComboBox->setCurrentItem(Kid3App::s_genCfg.m_importFormatIdx);
+	m_formatComboBox->QCM_addItems(Kid3App::s_genCfg.m_importFormatNames);
+	m_formatComboBox->QCM_setCurrentIndex(Kid3App::s_genCfg.m_importFormatIdx);
 	setFormatLineEdit(Kid3App::s_genCfg.m_importFormatIdx);
 
 	m_mismatchCheckBox->setChecked(Kid3App::s_genCfg.m_enableTimeDifferenceCheck);
@@ -329,15 +438,22 @@ void ImportSelector::fromFile()
 {
 	QString fn =
 #ifdef CONFIG_USE_KDE
-		KFileDialog::getOpenFileName(QString::null, QString::null, this);
+		KFileDialog::getOpenFileName(getImportDir(), QString::null, this);
 #else
-		QFileDialog::getOpenFileName(QString::null, QString::null, this);
+		QFileDialog::QCM_getOpenFileName(this, getImportDir());
 #endif
 	if (!fn.isEmpty()) {
 		QFile file(fn);
 		if (file.open(QCM_ReadOnly)) {
+			setImportDir(
+#if QT_VERSION >= 0x040000
+				QFileInfo(file).dir().path()
+#else
+				QFileInfo(file).dirPath(true)
+#endif
+				);
 			QTextStream stream(&file);
-			m_text = stream.read();
+			m_text = stream.QCM_readAll();
 			if (!m_text.isNull()) {
 				updateTrackData(File);
 				showPreview();
@@ -373,6 +489,32 @@ void ImportSelector::fromClipboard()
 #endif
 }
 
+/**
+ * Import from server and preview in table.
+ */
+void ImportSelector::fromServer()
+{
+	if (m_serverComboBox) {
+		switch (m_serverComboBox->QCM_currentIndex()) {
+			case ImportConfig::ServerFreedb:
+				fromFreedb();
+				break;
+			case ImportConfig::ServerTrackType:
+				fromTrackType();
+				break;
+			case ImportConfig::ServerDiscogs:
+				fromDiscogs();
+				break;
+			case ImportConfig::ServerMusicBrainzRelease:
+				fromMusicBrainzRelease();
+				break;
+			case ImportConfig::ServerMusicBrainzFingerprint:
+				fromMusicBrainz();
+				break;
+		}
+	}
+}
+
 /**
  * Import from freedb.org and preview in table.
  */
@@ -496,10 +638,10 @@ bool ImportSelector::updateTrackData(ImportSource impSrc) {
 
 	if (!start) {
 		/* start is false => tags were found */
-		Q3ValueList<int>* trackDuration = getTrackDurations();
+		TrackDurationList* trackDuration = getTrackDurations();
 		if (trackDuration) {
 			it = m_trackDataVector.begin();
-			for (Q3ValueList<int>::const_iterator tdit = trackDuration->begin();
+			for (TrackDurationList::const_iterator tdit = trackDuration->begin();
 					 tdit != trackDuration->end();
 					 ++tdit) {
 				if (it != m_trackDataVector.end()) {
@@ -519,9 +661,115 @@ bool ImportSelector::updateTrackData(ImportSource impSrc) {
  * Show fields to import in text as preview in table.
  */
 void ImportSelector::showPreview() {
+#if QT_VERSION >= 0x040000
+  m_tab->setRowCount(m_trackDataVector.size());
+	int row = 0;
+	QStringList vLabels;
+	QString str;
+	for (ImportTrackDataVector::const_iterator it = m_trackDataVector.begin();
+			 it != m_trackDataVector.end();
+			 ++it) {
+		QTableWidgetItem* twi;
+		int fileDuration = (*it).getFileDuration();
+		if (fileDuration != 0) {
+			vLabels.append(TaggedFile::formatTime(fileDuration));
+		}
+		str.clear();
+		int importDuration = (*it).getImportDuration();
+		if (importDuration != 0) {
+			str = TaggedFile::formatTime(importDuration);
+		}
+		if ((twi = m_tab->item(row, LengthColumn)) != 0) {
+			twi->setText(str);
+		} else {
+			twi = new QTableWidgetItem(str);
+			twi->setFlags(twi->flags() & ~Qt::ItemIsEditable);
+			m_tab->setItem(row, LengthColumn, twi);
+		}
+		str.clear();
+		if ((*it).track != -1) {
+			str.setNum((*it).track);
+		}
+		if ((twi = m_tab->item(row, TrackColumn)) != 0) {
+			twi->setText(str);
+		} else {
+			twi = new QTableWidgetItem(str);
+			twi->setFlags(twi->flags() & ~Qt::ItemIsEditable);
+			m_tab->setItem(row, TrackColumn, twi);
+		}
+		str.clear();
+		if (!(*it).title.isNull()) {
+			str = (*it).title;
+		}
+		if ((twi = m_tab->item(row, TitleColumn)) != 0) {
+			twi->setText(str);
+		} else {
+			twi = new QTableWidgetItem(str);
+			twi->setFlags(twi->flags() & ~Qt::ItemIsEditable);
+			m_tab->setItem(row, TitleColumn, twi);
+		}
+		str.clear();
+		if (!(*it).artist.isNull()) {
+			str = (*it).artist;
+		}
+		if ((twi = m_tab->item(row, ArtistColumn)) != 0) {
+			twi->setText(str);
+		} else {
+			twi = new QTableWidgetItem(str);
+			twi->setFlags(twi->flags() & ~Qt::ItemIsEditable);
+			m_tab->setItem(row, ArtistColumn, twi);
+		}
+		str.clear();
+		if (!(*it).album.isNull()) {
+			str = (*it).album;
+		}
+		if ((twi = m_tab->item(row, AlbumColumn)) != 0) {
+			twi->setText(str);
+		} else {
+			twi = new QTableWidgetItem(str);
+			twi->setFlags(twi->flags() & ~Qt::ItemIsEditable);
+			m_tab->setItem(row, AlbumColumn, twi);
+		}
+		str.clear();
+		if ((*it).year != -1) {
+			str.setNum((*it).year);
+		}
+		if ((twi = m_tab->item(row, YearColumn)) != 0) {
+			twi->setText(str);
+		} else {
+			twi = new QTableWidgetItem(str);
+			twi->setFlags(twi->flags() & ~Qt::ItemIsEditable);
+			m_tab->setItem(row, YearColumn, twi);
+		}
+		str.clear();
+		if (!(*it).genre.isNull()) {
+			str = (*it).genre;
+		}
+		if ((twi = m_tab->item(row, GenreColumn)) != 0) {
+			twi->setText(str);
+		} else {
+			twi = new QTableWidgetItem(str);
+			twi->setFlags(twi->flags() & ~Qt::ItemIsEditable);
+			m_tab->setItem(row, GenreColumn, twi);
+		}
+		str.clear();
+		if (!(*it).comment.isNull()) {
+			str = (*it).comment;
+		}
+		if ((twi = m_tab->item(row, CommentColumn)) != 0) {
+			twi->setText(str);
+		} else {
+			twi = new QTableWidgetItem(str);
+			twi->setFlags(twi->flags() & ~Qt::ItemIsEditable);
+			m_tab->setItem(row, CommentColumn, twi);
+		}
+		++row;
+	}
+	m_tab->setVerticalHeaderLabels(vLabels);
+#else
 	m_tab->setNumRows(0);
 	int row = 0;
-	Q3Header* vHeader = m_tab->verticalHeader();
+	QHeader* vHeader = m_tab->verticalHeader();
 	for (ImportTrackDataVector::const_iterator it = m_trackDataVector.begin();
 			 it != m_trackDataVector.end();
 			 ++it) {
@@ -555,6 +803,7 @@ void ImportSelector::showPreview() {
 			m_tab->setText(row, CommentColumn, (*it).comment);
 		++row;
 	}
+#endif
 
 	// make time difference check
 	m_tab->clearMarks();
@@ -610,11 +859,12 @@ bool ImportSelector::getNextTags(StandardTags& st, bool start)
 /**
  * Get import destination.
  *
- * @return DestV1 or DestV2 for ID3v1 or ID3v2.
+ * @return DestV1, DestV2 or DestV1V2 for ID3v1, ID3v2 or both.
  */
-ImportSelector::Destination ImportSelector::getDestination()
+ImportConfig::ImportDestination ImportSelector::getDestination()
 {
-	return (Destination)m_destComboBox->currentItem();
+	return static_cast<ImportConfig::ImportDestination>(
+		m_destComboBox->QCM_currentIndex());
 }
 
 /**
@@ -622,10 +872,12 @@ ImportSelector::Destination ImportSelector::getDestination()
  */
 void ImportSelector::saveConfig()
 {
-	Kid3App::s_genCfg.m_importDestV1 = 
-		(m_destComboBox->currentItem() == static_cast<int>(ImportSelector::DestV1));
+	Kid3App::s_genCfg.m_importDest = static_cast<ImportConfig::ImportDestination>(
+		m_destComboBox->QCM_currentIndex());
 
-	Kid3App::s_genCfg.m_importFormatIdx = m_formatComboBox->currentItem();
+	Kid3App::s_genCfg.m_importServer = static_cast<ImportConfig::ImportServer>(
+		m_serverComboBox->QCM_currentIndex());
+	Kid3App::s_genCfg.m_importFormatIdx = m_formatComboBox->QCM_currentIndex();
 	Kid3App::s_genCfg.m_importFormatNames[Kid3App::s_genCfg.m_importFormatIdx] = m_formatComboBox->currentText();
 	Kid3App::s_genCfg.m_importFormatHeaders[Kid3App::s_genCfg.m_importFormatIdx] = m_headerLineEdit->text();
 	Kid3App::s_genCfg.m_importFormatTracks[Kid3App::s_genCfg.m_importFormatIdx] = m_trackLineEdit->text();
@@ -640,9 +892,9 @@ void ImportSelector::saveConfig()
  * @return list with track durations,
  *         0 if no track durations found.
  */
-Q3ValueList<int>* ImportSelector::getTrackDurations()
+TrackDurationList* ImportSelector::getTrackDurations()
 {
-	Q3ValueList<int>* lst = 0;
+	TrackDurationList* lst = 0;
 	if (m_headerParser && ((lst = m_headerParser->getTrackDurations()) != 0) &&
 			(lst->size() > 0)) {
 		return lst;
@@ -683,9 +935,19 @@ void ImportSelector::maxDiffChanged() {
  * @param fromIndex index of position moved to
  */
 void ImportSelector::moveTableRow(int, int fromIndex, int toIndex) {
+#if QT_VERSION >= 0x040000
+	QHeaderView* vHeader = m_tab->verticalHeader();
+	if (vHeader) {
+		// revert movement, but avoid recursion
+		disconnect(vHeader, SIGNAL(sectionMoved(int, int, int)), 0, 0);
+		vHeader->moveSection(toIndex, fromIndex);
+		connect(vHeader, SIGNAL(sectionMoved(int, int, int)), this, SLOT(moveTableRow(int, int, int)));
+	}
+#else
 	if (toIndex > fromIndex && toIndex > 0) {
 		--toIndex;
 	}
+#endif
 	int numTracks = static_cast<int>(m_trackDataVector.size());
 	if (fromIndex < numTracks && toIndex < numTracks) {
 		// swap elements but keep file durations and names
@@ -956,21 +1218,21 @@ void ImportSelector::matchWithTitle()
 			QString fileName = (*it).getAbsFilename();
 			if (!fileName.isEmpty()) {
 				++numFiles;
-				int startIndex = fileName.findRev(QDir::separator()) + 1;
-				int endIndex = fileName.findRev('.');
+				int startIndex = fileName.QCM_lastIndexOf(QDir::separator()) + 1;
+				int endIndex = fileName.QCM_lastIndexOf('.');
 				if (endIndex > startIndex) {
 					fileName = fileName.mid(startIndex, endIndex - startIndex);
 				} else {
 					fileName = fileName.mid(startIndex);
 				}
-				md[i].fileWords = QStringList::split(
-					nonWordCharRegExp, fileName.lower().
+				md[i].fileWords = QCM_split(
+					nonWordCharRegExp, fileName.QCM_toLower().
 					replace(nonLetterSpaceRegExp, " "));
 			}
 			if (!(*it).title.isEmpty()) {
 				++numImports;
-				md[i].titleWords = QStringList::split(
-					nonWordCharRegExp, (*it).title.lower().
+				md[i].titleWords = QCM_split(
+					nonWordCharRegExp, (*it).title.QCM_toLower().
 					replace(nonLetterSpaceRegExp, " "));
 			}
 			md[i].assignedTo = -1;
