@@ -5,20 +5,37 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 17 Sep 2003
+ *
+ * Copyright (C) 2003-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef IMPORTSELECTOR_H
 #define IMPORTSELECTOR_H
 
 #include "importtrackdata.h"
+#include "importparser.h"
+#include "importconfig.h"
+#include <qwidget.h>
 #include <qstring.h>
 #include <qstringlist.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
-#include <Q3VBox>
-#include <Q3ValueList>
-#else
-#include <qvbox.h>
+#include <QList>
 #endif
 
 class QPushButton;
@@ -38,30 +55,32 @@ class ImportTable;
 /**
  * Import selector widget.
  */
-class ImportSelector : public Q3VBox
+class ImportSelector : public QWidget
 {
 Q_OBJECT
 
 public:
-	/** Import destinations */
-	enum Destination { DestV1, DestV2 };
-
 	/**
 	 * Constructor.
 	 *
 	 * @param parent        parent widget
 	 * @param trackDataList track data to be filled with imported values,
 	 *                      is passed with durations of files set
-	 * @param name          Qt name
-	 * @param f             window flags
 	 */
-	ImportSelector(QWidget* parent, ImportTrackDataVector& trackDataList,
-								 const char* name = 0, Qt::WFlags f = 0);
+	ImportSelector(QWidget* parent, ImportTrackDataVector& trackDataList);
+
 	/**
 	 * Destructor.
 	 */
 	~ImportSelector();
 
+	/**
+	 * Select the import server.
+	 *
+	 * @param server import server
+	 */
+	void setImportServer(ImportConfig::ImportServer server);
+
 	/**
 	 * Clear dialog data.
 	 */
@@ -93,9 +112,9 @@ public:
 	/**
 	 * Get import destination.
 	 *
-	 * @return DestV1 or DestV2 for ID3v1 or ID3v2.
+	 * @return DestV1, DestV2 or DestV1V2 for ID3v1, ID3v2 or both.
 	 */
-	Destination getDestination();
+	ImportConfig::ImportDestination getDestination();
 
 	/**
 	 * Get list with track durations.
@@ -103,7 +122,7 @@ public:
 	 * @return list with track durations,
 	 *         0 if no track durations found.
 	 */
-	Q3ValueList<int>* getTrackDurations();
+	TrackDurationList* getTrackDurations();
 
 	/**
 	 * Get time difference check configuration.
@@ -126,6 +145,18 @@ public:
 	 */
 	static const char** s_lineFmtList;
 
+	/**
+	 * Get last directory used for import or export.
+	 * @return import directory.
+	 */
+	static QString getImportDir() { return s_importDir; }
+
+	/**
+	 * Set last directory used for import or export.
+	 * @param dir import directory
+	 */
+	static void setImportDir(const QString& dir) { s_importDir = dir; }
+
 public slots:
 	/**
 	 * Called when the maximum time difference value is changed.
@@ -152,6 +183,11 @@ public slots:
 	 */
 	void fromClipboard();
 
+	/**
+	 * Import from server and preview in table.
+	 */
+	void fromServer();
+
 	/**
 	 * Import from freedb.org and preview in table.
 	 */
@@ -232,16 +268,8 @@ private:
 	QPushButton* m_fileButton;
 	/** From Clipboard button */
 	QPushButton* m_clipButton;
-	/** From freedb.org button */
-	QPushButton* m_freedbButton;
-	/** From TrackType.org button */
-	QPushButton* m_trackTypeButton;
-	/** From MusicBrainz button */
-	QPushButton* m_musicBrainzButton;
-	/** From MusicBrainz Release button */
-	QPushButton* m_musicBrainzReleaseButton;
-	/** From Discogs button */
-	QPushButton* m_discogsButton;
+	/** From Server button */
+	QPushButton* m_serverButton;
 	/** Match with Length button */
 	QPushButton* m_lengthButton;
 	/** Match with Track button */
@@ -252,6 +280,8 @@ private:
 	ImportTable* m_tab;
 	/** contents of imported file/clipboard */
 	QString m_text;
+	/** combobox with import servers */
+	QComboBox* m_serverComboBox;
 	/** combobox with import destinations */
 	QComboBox* m_destComboBox;
 	/** combobox with import formats */
@@ -284,6 +314,9 @@ private:
 	ImportSource m_importSource;
 	/** track data */
 	ImportTrackDataVector& m_trackDataVector;
+
+	/** Last directory used for import or export. */
+	static QString s_importDir;
 };
 
 #endif
