@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 09 Oct 2006
+ *
+ * Copyright (C) 2006-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef IMPORTSOURCEDIALOG_H
@@ -16,8 +33,7 @@
 #include <qstring.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
-#include <Q3ListBox>
-#include <Q3CString>
+#include <QListWidget>
 #else
 #include <qlistbox.h>
 #endif
@@ -26,8 +42,6 @@ class QLineEdit;
 class QComboBox;
 class QPushButton;
 class QCheckBox;
-class Q3ListBox;
-class Q3ListBoxItem;
 class QStatusBar;
 class ImportSourceConfig;
 class ImportSourceClient;
@@ -153,7 +167,7 @@ private slots:
 	 * @param li list box item containing an AlbumListItem
 	 */
 #if QT_VERSION >= 0x040000
-	void requestTrackList(Q3ListBoxItem* li);
+	void requestTrackList(QListWidgetItem* li);
 #else
 	void requestTrackList(QListBoxItem* li);
 #endif
@@ -182,7 +196,11 @@ signals:
 	void trackDataUpdated();
 
 protected:
-	Q3ListBox* m_albumListBox; /**< list box with albums to select */
+#if QT_VERSION >= 0x040000
+	QListWidget* m_albumListBox; /**< list box with albums to select */
+#else
+	QListBox* m_albumListBox; /**< list box with albums to select */
+#endif
 	ImportTrackDataVector& m_trackDataVector; /**< vector with tracks to import */
 
 private:
@@ -203,10 +221,16 @@ private:
 	const Properties& m_props;
 };
 
+#if QT_VERSION >= 0x040000
+typedef QListWidgetItem AlbumListItemBaseClass;
+#else
+typedef QListBoxText AlbumListItemBaseClass;
+#endif
+
 /**
  * QListBoxItem subclass for album list.
  */
-class AlbumListItem : public Q3ListBoxText {
+class AlbumListItem : public AlbumListItemBaseClass {
 public:
 	/**
 	 * Constructor.
@@ -215,9 +239,15 @@ public:
 	 * @param cat     category
 	 * @param idStr   ID
 	 */
-	AlbumListItem(Q3ListBox* listbox, const QString& text,
+#if QT_VERSION >= 0x040000
+	AlbumListItem(QListWidget* listbox, const QString& text,
 				  const QString& cat, const QString& idStr) : 
-		Q3ListBoxText(listbox, text), m_category(cat), m_id(idStr) {}
+		QListWidgetItem(text, listbox), m_category(cat), m_id(idStr) {}
+#else
+	AlbumListItem(QListBox* listbox, const QString& text,
+				  const QString& cat, const QString& idStr) : 
+		QListBoxText(listbox, text), m_category(cat), m_id(idStr) {}
+#endif
 
 	/**
 	 * Get category.
