@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 23 Feb 2007
+ *
+ * Copyright (C) 2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "importtrackdata.h"
@@ -38,9 +55,9 @@ QString ImportTrackData::formatString(const QString& format) const
 		QString tagStr[numTagCodes];
 
 		QString filename(getAbsFilename());
-		int sepPos = filename.findRev('/');
+		int sepPos = filename.QCM_lastIndexOf('/');
 		if (sepPos < 0) {
-			sepPos = filename.findRev(QDir::separator());
+			sepPos = filename.QCM_lastIndexOf(QDir::separator());
 		}
 		if (sepPos >= 0) {
 			filename.remove(0, sepPos + 1);
@@ -49,8 +66,8 @@ QString ImportTrackData::formatString(const QString& format) const
 		tagStr[0] = filename;
 		tagStr[1] = getAbsFilename();
 		QUrl url;
-		url.setFileName(tagStr[1]);
-		url.setProtocol("file");
+		url.QCM_setPath(tagStr[1]);
+		url.QCM_setScheme("file");
 		tagStr[2] = url.toString(
 #if QT_VERSION < 0x040000
 			true
