@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 7 Jul 2005
+ *
+ * Copyright (C) 2005-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef IMPORTTRACKDATA_H
@@ -14,7 +31,7 @@
 #include <qglobal.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
-#include <Q3ValueVector>
+#include <QVector>
 #else
 #include <qvaluevector.h>
 #endif
@@ -103,7 +120,13 @@ private:
 /**
  * Vector containing tracks to import and artist, album names.
  */
-class ImportTrackDataVector : public Q3ValueVector<ImportTrackData> {
+class ImportTrackDataVector : public
+#if QT_VERSION >= 0x040000
+QVector<ImportTrackData>
+#else
+QValueVector<ImportTrackData>
+#endif
+{
 public:
 	QString m_artist; /**< album artist */
 	QString m_album;  /**< album name */
