@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 9 Jan 2003
+ *
+ * Copyright (C) 2003-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef KID3_H
@@ -12,25 +29,26 @@
 
 #include "config.h"
 #ifdef CONFIG_USE_KDE
-#include <kmainwindow.h>
+#include <kdeversion.h>
 class KAction;
 class KRecentFilesAction;
 class KToggleAction;
 /** Base class for main window. */
+#if KDE_VERSION >= 0x035c00
+#include <kxmlguiwindow.h>
+typedef KXmlGuiWindow Kid3AppBaseClass;
+#else
+#include <kmainwindow.h>
 typedef KMainWindow Kid3AppBaseClass;
+#endif
 #else
 #include "qtcompatmac.h"
-#if QT_VERSION >= 0x040000
-#include <Q3MainWindow>
-#else
 #include <qmainwindow.h>
-#endif
 #include "generalconfig.h" // Kid3Settings
 class QAction;
 class BrowserDialog;
-class Q3PopupMenu;
 /** Base class for main window. */
-typedef Q3MainWindow Kid3AppBaseClass;
+typedef QMainWindow Kid3AppBaseClass;
 #endif
 #include "importtrackdata.h"
 #include "formatconfig.h"
@@ -39,8 +57,10 @@ typedef Q3MainWindow Kid3AppBaseClass;
 #include "freedbconfig.h"
 #include "discogsconfig.h"
 #include "musicbrainzconfig.h"
+#include "frame.h"
 
 class KURL;
+class KUrl;
 class Id3Form;
 class TaggedFile;
 class StandardTags;
@@ -69,7 +89,7 @@ public:
 	 * Open directory.
 	 *
 	 * @param dir     directory or file path
- 	 * @param confirm if true ask if there are unsaved changes
+	 * @param confirm if true ask if there are unsaved changes
 	 */
 	void openDirectory(QString dir, bool confirm = false);
 
@@ -87,18 +107,14 @@ public:
 	void updateCurrentSelection();
 
 	/**
-	 * Copy a set of standard tags into copy buffer.
-	 *
-	 * @param st tags to copy
+	 * Copy tags 1 into copy buffer.
 	 */
-	void copyTags(const StandardTags* st);
+	void copyTagsV1();
 
 	/**
-	 * Paste from copy buffer to standard tags.
-	 *
-	 * @param st tags to fill from data in copy buffer.
+	 * Copy tags 2 into copy buffer.
 	 */
-	void pasteTags(StandardTags* st);
+	void copyTagsV2();
 
 	/**
 	 * Paste from copy buffer to ID3v1 tags.
@@ -182,6 +198,12 @@ public:
 	 */
 	static void displayHelp(const QString& anchor = QString::null);
 
+	/**
+	 * Get directory name.
+	 * @return directory.
+	 */
+	static QString getDirName() { return s_dirName; }
+
 	/** Filename format configuration */
 	static FormatConfig s_fnFormatCfg;
 	/** ID3 format configuration */
@@ -205,6 +227,11 @@ protected:
 	 */
 	void initActions();
 
+	/**
+	 * Init file types.
+	 */
+	void initFileTypes();
+
 	/**
 	 * Init status bar.
 	 */
@@ -221,6 +248,16 @@ protected:
 	 */
 	void cleanup();
 
+	/**
+	 * Create a filter string for the file dialog.
+	 * The filter string contains entries for all supported types.
+	 *
+	 * @param defaultNameFilter if not 0, return default name filter here
+	 *
+	 * @return filter string.
+	 */
+	QString createFilterString(QString* defaultNameFilter = 0) const;
+
 	/**
 	 * Update modification state before closing.
 	 * Called on closeEvent() of window.
@@ -237,14 +274,26 @@ protected:
 	 *
 	 * @param cfg application configuration
 	 */
-	virtual void saveProperties(KConfig* cfg);
+	virtual void saveProperties(
+#if KDE_VERSION >= 0x035c00
+		KConfigGroup& cfg
+#else
+		KConfig* cfg
+#endif
+		);
 
 	/**
 	 * Reads the session config file and restores the application's state.
 	 *
 	 * @param cfg application configuration
 	 */
-	virtual void readProperties(KConfig* cfg);
+	virtual void readProperties(
+#if KDE_VERSION >= 0x035c00
+		KConfigGroup& cfg
+#else
+		KConfig* cfg
+#endif
+		);
 
 #else
 	/**
@@ -253,6 +302,11 @@ protected:
 	 * @param ce close event
 	 */
 	void closeEvent(QCloseEvent* ce);
+
+	/**
+	 * Read font and style options.
+	 */
+	void readFontAndStyleOptions();
 #endif
 
 	/**
@@ -285,6 +339,11 @@ public slots:
 	 */
 	void slotFileOpen();
 
+	/**
+	 * Request new directory and open it.
+	 */
+	void slotFileOpenDirectory();
+
 	/**
 	 * Open recent directory.
 	 *
@@ -292,6 +351,13 @@ public slots:
 	 */
 	void slotFileOpenRecent(const KURL& url);
 
+	/**
+	 * Open recent directory.
+	 *
+	 * @param url URL of directory to open
+	 */
+	void slotFileOpenRecentUrl(const KUrl& url);
+
 	/**
 	 * Quit application.
 	 */
@@ -307,6 +373,11 @@ public slots:
 	 */
 	void slotSettingsShortcuts();
 
+	/**
+	 * Toolbars configuration.
+	 */
+	void slotSettingsToolbars();
+
 	/**
 	 * Display handbook.
 	 */
@@ -481,16 +552,12 @@ private:
 	void updateAfterFrameModification(TaggedFile* taggedFile);
 
 	/**
-	 * Get the selected file together with its frame list.
-	 * If multiple files are selected, 0 is returned for both parameters.
+	 * Get the selected file.
 	 *
-	 * @param taggedFile the file is returned here,
-	 *                   0 if not exactly one file is selected
-	 * @param framelist  the frame list is returned here,
-	 *                   0 if not exactly one file is selected
+	 * @return the selected file,
+	 *         0 if not exactly one file is selected
 	 */
-	void getSelectedFileWithFrameList(
-		TaggedFile*& taggedFile, FrameList*& framelist);
+	TaggedFile* getSelectedFile();
 	
 	/**
 	 * Format a filename if format while editing is switched on.
@@ -506,6 +573,13 @@ private:
 	 */
 	void formatStandardTagsIfEnabled(StandardTags* st) const;
 
+	/**
+	 * Format frames if format while editing is switched on.
+	 *
+	 * @param frames frames
+	 */
+	void formatFramesIfEnabled(FrameCollection& frames) const;
+
 	/**
 	 * Update track data and create import dialog.
 	 */
@@ -532,10 +606,8 @@ private:
 	Id3Form* m_view;
 	/** true if any file was modified */
 	bool m_modified;
-	/** Current directory */
-	QString m_dirName;
 	/** Copy buffer */
-	StandardTags* m_copyTags;
+	FrameCollection m_copyTags;
 	/** Import dialog */
 	ImportDialog* m_importDialog;
 	/** Track data list */
@@ -544,62 +616,28 @@ private:
 	ExportDialog* m_exportDialog;
 	/** Number tracks dialog */
 	NumberTracksDialog* m_numberTracksDialog;
+	/** Frame list */
+	FrameList* m_framelist;
 
 #ifdef CONFIG_USE_KDE
 	/** the configuration object of the application */
 	KConfig* m_config;
 	/** Actions */
-	KAction* m_fileOpen;
 	KRecentFilesAction* m_fileOpenRecent;
-	KAction* m_fileRevert;
-	KAction* m_fileSave;
-	KAction* m_fileQuit;
 	KToggleAction* m_viewToolBar;
 	KToggleAction* m_viewStatusBar;
-	KAction* settingsShortcuts;
-	KAction* m_settingsConfigure;
 	KAction* m_settingsShowHideV1;
 	KAction* m_settingsShowHideV2;
 #else
 	Kid3Settings* m_config;
-	QAction* m_fileOpen;
-	QAction* m_fileSave;
-	QAction* m_fileRevert;
-	QAction* m_fileCreatePlaylist;
-	QAction* m_fileQuit;
-	QAction* m_fileImport;
-	QAction* m_fileImportFreedb;
-	QAction* m_fileImportTrackType;
-	QAction* m_fileImportDiscogs;
-	QAction* m_fileImportMusicBrainzRelease;
-#ifdef HAVE_TUNEPIMP
-	QAction* m_fileImportMusicBrainz;
-#endif
-	QAction* m_fileExport;
-	QAction* m_toolsApplyFilenameFormat;
-	QAction* m_toolsApplyId3Format;
-	QAction* m_toolsRenameDirectory;
-	QAction* m_toolsNumberTracks;
-#ifdef HAVE_TAGLIB
-	QAction* m_toolsConvertToId3v24;
-#endif
-#if defined HAVE_TAGLIB && defined HAVE_ID3LIB
-	QAction* m_toolsConvertToId3v23;
-#endif
 	QAction* m_settingsShowHideV1;
 	QAction* m_settingsShowHideV2;
-	QAction* m_settingsConfigure;
-	QAction* m_helpHandbook;
-	QAction* m_helpAbout;
-	QAction* m_helpAboutQt;
-	QMenuBar* m_menubar;
-	Q3PopupMenu* m_fileMenu;
-	Q3PopupMenu* m_toolsMenu;
-	Q3PopupMenu* m_settingsMenu;
-	Q3PopupMenu* m_helpMenu;
 
 	static BrowserDialog* s_helpBrowser;
 #endif
+
+	/** Current directory */
+	static QString s_dirName;
 };
 
 #endif /* KID3_H */
