@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 17 Jan 2004
+ *
+ * Copyright (C) 2004-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MISCCONFIG_H
@@ -13,7 +30,7 @@
 #include <qstringlist.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
-#include <Q3ValueList>
+#include <QList>
 #else
 #include <qvaluelist.h>
 #endif
@@ -35,6 +52,13 @@ public:
 		ID3v2_4_0 = 1
 	};
 
+	/** Encoding used for ID3v2 frames. */
+	enum TextEncoding {
+		TE_ISO8859_1,
+		TE_UTF16,
+		TE_UTF8
+	};
+
 	/**
 	 * External command in context menu.
 	 */
@@ -151,14 +175,25 @@ public:
 	QString m_dirFormatText;
 	/** index of directory name format selected */
 	int m_dirFormatItem;
-	/** true to rename directory from ID3v1 tags, else ID3v2 tags */
-	bool m_renDirSrcV1;
+	/** rename directory from tags 1, tags 2, or both */
+	int m_renDirSrc;
+#if QT_VERSION >= 0x040000
+	/** List of splitter sizes. */
+	typedef QList<int> SizesList;
+	/** List of menu commands */
+	typedef QList<MenuCommand> MenuCommandList;
+#else
+	/** List of splitter sizes. */
+	typedef QValueList<int> SizesList;
+	/** List of menu commands */
+	typedef QValueList<MenuCommand> MenuCommandList;
+#endif
 	/** size of splitter in main window */
-	Q3ValueList<int> m_splitterSizes;
+	SizesList m_splitterSizes;
 	/** size of file/dirlist splitter */
-	Q3ValueList<int> m_vSplitterSizes;
+	SizesList m_vSplitterSizes;
 	/** commands available in context menu */
-	Q3ValueList<MenuCommand> m_contextMenuCommands;
+	MenuCommandList m_contextMenuCommands;
 	/** custom genres for ID3v2.3 */
 	QStringList m_customGenres;
 	/** true to hide ID3v1.1 controls */
@@ -167,6 +202,8 @@ public:
 	bool m_hideV2;
 	/** version used for new ID3v2 tags */
 	int m_id3v2Version;
+	/** text encoding used for new ID3v2 tags */
+	int m_textEncoding;
 	/** true if proxy is used */
 	bool m_useProxy;
 	/** proxy used for access */
@@ -180,10 +217,16 @@ public:
 	int m_windowWidth;
 	/** mainwindow height */
 	int m_windowHeight;
+	/** true if custom application font is used */
+	bool m_useFont;
+	/** custom application font family */
+	QString m_fontFamily;
+	/** custom application font size */
+	int m_fontSize;
+	/** custom application style, empty if not used */
+	QString m_style;
 #endif
 
-	/** Default name filter */
-	static const char* const s_defaultNameFilter;
 	/** Default value for comment name */
 	static const char* const s_defaultCommentName;
 	/** Default value for web browser */
