@@ -5,19 +5,34 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 26 Sep 2005
+ *
+ * Copyright (C) 2005-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "oggfile.h"
+#include "oggfile.hpp"
 #if defined HAVE_VORBIS || defined HAVE_FLAC
 
 #include "standardtags.h"
-#include "genres.h"
-#include "oggframelist.h"
 #include "dirinfo.h"
 #include <qfile.h>
 #include <qdir.h>
 #if QT_VERSION >= 0x040000
-#include <Q3CString>
+#include <QByteArray>
 #endif
 #include <sys/stat.h>
 #ifdef WIN32
@@ -62,7 +77,7 @@ void OggFile::readTags(bool force)
 		m_comments.clear();
 		markTag2Changed(false);
 		m_fileRead = true;
-		Q3CString fnIn = QFile::encodeName(getDirInfo()->getDirname() + QDir::separator() + currentFilename());
+		QCM_QCString fnIn = QFile::encodeName(getDirInfo()->getDirname() + QDir::separator() + currentFilename());
 
 		if (m_fileInfo.read(fnIn)) {
 			FILE* fpIn = ::fopen(fnIn, "rb");
@@ -76,12 +91,12 @@ void OggFile::readTags(bool force)
 								QString userComment =
 									QString::fromUtf8(vc->user_comments[i],
 																		vc->comment_lengths[i]);
-								int equalPos = userComment.find('=');
+								int equalPos = userComment.QCM_indexOf('=');
 								if (equalPos != -1) {
 									QString name(
-										userComment.left(equalPos).stripWhiteSpace().upper());
+										userComment.left(equalPos).QCM_trimmed().QCM_toUpper());
 									QString value(
-										userComment.mid(equalPos + 1).stripWhiteSpace());
+										userComment.mid(equalPos + 1).QCM_trimmed());
 									if (!value.isEmpty()) {
 										m_comments.push_back(CommentField(name, value));
 									}
@@ -127,10 +142,10 @@ bool OggFile::writeTags(bool force, bool* renamed, bool preserve)
 		if (!renameFile(currentFilename(), tempFilename)) {
 			return false;
 		}
-		Q3CString fnIn = QFile::encodeName(dirname + QDir::separator() +
-																			 tempFilename);
-		Q3CString fnOut = QFile::encodeName(dirname + QDir::separator() +
-																			 getFilename());
+		QCM_QCString fnIn = QFile::encodeName(dirname + QDir::separator() +
+																					tempFilename);
+		QCM_QCString fnOut = QFile::encodeName(dirname + QDir::separator() +
+																					 getFilename());
 		FILE* fpIn = ::fopen(fnIn, "rb");
 		if (fpIn) {
 
@@ -165,8 +180,8 @@ bool OggFile::writeTags(bool force, bool* renamed, bool preserve)
 								if (!value.isEmpty()) {
 									::vorbis_comment_add_tag(
 										vc,
-										const_cast<char*>(name.latin1()),
-										const_cast<char*>((const char*)value.utf8()));
+										const_cast<char*>(name.QCM_latin1()),
+										const_cast<char*>((const char*)value.QCM_toUtf8().data()));
 									++it;
 								} else {
 									it = m_comments.erase(it);
@@ -213,17 +228,101 @@ bool OggFile::writeTags(bool, bool*, bool) { return false; }
 #endif // HAVE_VORBIS
 
 /**
- * Remove all ID3v2 tags.
+ * Get name of frame from type.
  *
- * @param flt filter specifying which fields to remove
+ * @param type type
+ *
+ * @return name.
  */
-void OggFile::removeTagsV2(const StandardTagsFilter& flt)
+static const char* getVorbisNameFromType(Frame::Type type)
 {
-	if (flt.areAllTrue()) {
+  static const char* const names[] = {
+		"TITLE",           // FT_Title,
+		"ARTIST",          // FT_Artist,
+		"ALBUM",           // FT_Album,
+		"COMMENT",         // FT_Comment,
+		"DATE",            // FT_Date,
+		"TRACKNUMBER",     // FT_Track,
+		"GENRE",           // FT_Genre,
+		                   // FT_LastV1Frame = FT_Track,
+		"ARRANGER",        // FT_Arranger,
+		"AUTHOR",          // FT_Author,
+		"BPM",             // FT_Bpm,
+		"COMPOSER",        // FT_Composer,
+		"CONDUCTOR",       // FT_Conductor,
+		"COPYRIGHT",       // FT_Copyright,
+		"DISCNUMBER",      // FT_Disc,
+		"ENCODED-BY",      // FT_EncodedBy,
+		"ISRC",            // FT_Isrc,
+		"LANGUAGE",        // FT_Language,
+		"LYRICIST",        // FT_Lyricist,
+		"ORIGINALALBUM",   // FT_OriginalAlbum,
+		"ORIGINALARTIST",  // FT_OriginalArtist,
+		"ORIGINALDATE",    // FT_OriginalDate,
+		"PART",            // FT_Part,
+		"PERFORMER",       // FT_Performer,
+		"PUBLISHER",       // FT_Publisher,
+		"SUBTITLE",        // FT_Subtitle,
+		"WEBSITE",         // FT_Website,
+		                   // FT_LastFrame = FT_Website
+	};
+	class not_used { int array_size_check[
+			sizeof(names) / sizeof(names[0]) == Frame::FT_LastFrame + 1
+			? 1 : -1 ]; };
+	return type <= Frame::FT_LastFrame ? names[type] : "UNKNOWN";
+}
+
+/**
+ * Get the frame type for a Vorbis name.
+ *
+ * @param name Vorbis tag name
+ *
+ * @return frame type.
+ */
+static Frame::Type getTypeFromVorbisName(QString name)
+{
+	static QMap<QString, int> strNumMap;
+	if (strNumMap.empty()) {
+		// first time initialization
+		for (int i = 0; i <= Frame::FT_LastFrame; ++i) {
+			Frame::Type type = static_cast<Frame::Type>(i);
+			strNumMap.insert(getVorbisNameFromType(type), type);
+		}
+		strNumMap.insert("DESCRIPTION", Frame::FT_Comment);
+	}
+	QMap<QString, int>::const_iterator it =
+		strNumMap.find(name.remove(' ').QCM_toUpper());
+	if (it != strNumMap.end()) {
+		return static_cast<Frame::Type>(*it);
+	}
+	return Frame::FT_Other;
+}
+
+/**
+ * Remove ID3v2 frames.
+ *
+ * @param flt filter specifying which frames to remove
+ */
+void OggFile::deleteFramesV2(const FrameFilter& flt)
+{
+	if (flt.areAllEnabled()) {
 		m_comments.clear();
 		markTag2Changed();
 	} else {
-		removeStandardTagsV2(flt);
+		bool changed = false;
+		for (OggFile::CommentList::iterator it = m_comments.begin();
+				 it != m_comments.end();) {
+			QString name((*it).getName());
+			if (flt.isEnabled(getTypeFromVorbisName(name), name)) {
+				it = m_comments.erase(it);
+				changed = true;
+			} else {
+				++it;
+			}
+		}
+		if (changed) {
+			markTag2Changed();
+		}
 	}
 }
 
@@ -303,7 +402,7 @@ int OggFile::getTrackNumV2()
 	if (str.isNull()) return -1;
 	if (str.isEmpty()) return 0;
 	// handle "track/total number of tracks" format
-	int slashPos = str.find('/');
+	int slashPos = str.QCM_indexOf('/');
 	if (slashPos != -1) {
 		str.truncate(slashPos);
 	}
@@ -524,31 +623,152 @@ QString OggFile::getTagFormatV2() const
 	return hasTagV2() ? QString("Vorbis") : QString::null;
 }
 
-/** Frame list for Ogg files. */
-OggFrameList* OggFile::s_oggFrameList = 0;
-
 /**
- * Get frame list for this type of tagged file.
+ * Set a frame in the tags 2.
  *
- * @return frame list.
+ * @param frame frame to set
+ *
+ * @return true if ok.
  */
-FrameList* OggFile::getFrameList() const
+bool OggFile::setFrameV2(const Frame& frame)
 {
-	if (!s_oggFrameList) {
-		s_oggFrameList = new OggFrameList();
+	if (frame.getType() == Frame::FT_Track) {
+		int numTracks = getTotalNumberOfTracksIfEnabled();
+		if (numTracks > 0) {
+			setTextField("TRACKTOTAL", QString::number(numTracks));
+		}
 	}
-	return s_oggFrameList;
+
+	// If the frame has an index, change that specific frame
+	int index = frame.getIndex();
+	if (index != -1 && index < static_cast<int>(m_comments.size())) {
+#if QT_VERSION >= 0x040000
+		m_comments[index].setValue(frame.getValue());
+		return true;
+#else
+		CommentList::iterator it = m_comments.at(index);
+		if (it != m_comments.end()) {
+			(*it).setValue(frame.getValue());
+			return true;
+		}
+#endif
+	}
+
+	// Try the superclass method
+	return TaggedFile::setFrameV2(frame);
 }
 
 /**
- * Clean up static resources.
+ * Add a frame in the tags 2.
+ *
+ * @param frame frame to add
+ *
+ * @return true if ok.
  */
-void OggFile::staticCleanup()
+bool OggFile::addFrameV2(Frame& frame)
 {
-	delete s_oggFrameList;
-	s_oggFrameList = 0;
+	// Add a new frame.
+	QString name(frame.getName().remove(' ').QCM_toUpper());
+	m_comments.push_back(OggFile::CommentField(name, frame.getValue()));
+	frame.setInternalName(name);
+	frame.setIndex(m_comments.size() - 1);
+	markTag2Changed();
+	return true;
 }
 
+/**
+ * Delete a frame in the tags 2.
+ *
+ * @param frame frame to delete.
+ *
+ * @return true if ok.
+ */
+bool OggFile::deleteFrameV2(const Frame& frame)
+{
+	// If the frame has an index, delete that specific frame
+	int index = frame.getIndex();
+	if (index != -1 && index < static_cast<int>(m_comments.size())) {
+#if QT_VERSION >= 0x040000
+		m_comments.removeAt(index);
+#else
+		OggFile::CommentList::iterator it = m_comments.at(index);
+		m_comments.erase(it);
+#endif
+		markTag2Changed();
+		return true;
+	}
+
+	// Try the superclass method
+	return TaggedFile::deleteFrameV2(frame);
+}
+
+/**
+ * Get all frames in tag 2.
+ *
+ * @param frames frame collection to set.
+ */
+void OggFile::getAllFramesV2(FrameCollection& frames)
+{
+	frames.clear();
+	QString name;
+	int i = 0;
+	for (OggFile::CommentList::const_iterator it = m_comments.begin();
+			 it != m_comments.end();
+			 ++it) {
+		name = (*it).getName();
+		Frame::Type type = getTypeFromVorbisName(name);
+		frames.insert(Frame(type, (*it).getValue(), name, i++));
+	}
+	frames.addMissingStandardFrames();
+}
+
+/**
+ * Get a list of frame IDs which can be added.
+ *
+ * @return list with frame IDs.
+ */
+QStringList OggFile::getFrameIds() const
+{
+	static const char* const fieldNames[] = {
+		"ALBUMARTIST",
+		"CATALOGNUMBER",
+		"CONTACT",
+		"DESCRIPTION",
+		"EAN/UPN",
+		"ENCODING",
+		"ENGINEER",
+		"ENSEMBLE",
+		"GUEST ARTIST",
+		"LABEL",
+		"LABELNO",
+		"LICENSE",
+		"LOCATION",
+		"OPUS",
+		"ORGANIZATION",
+		"PARTNUMBER",
+		"PRODUCER",
+		"PRODUCTNUMBER",
+		"RECORDINGDATE",
+		"RELEASE DATE",
+		"REMIXER",
+		"SOURCE ARTIST",
+		"SOURCE MEDIUM",
+		"SOURCE WORK",
+		"SOURCEMEDIA",
+		"SPARS",
+		"TRACKTOTAL",
+		"VERSION",
+		"VOLUME"
+	};
+
+	QStringList lst(TaggedFile::getFrameIds());
+	for (unsigned i = 0; i < sizeof(fieldNames) / sizeof(fieldNames[0]); ++i) {
+		lst.append(fieldNames[i]);
+	}
+	return lst;
+}
+
+
 
 /**
  * Read information about an Ogg/Vorbis file.
@@ -591,8 +811,6 @@ bool OggFile::FileInfo::read(const char* fn)
 #else // HAVE_VORBIS
 QString OggFile::getDetailInfo() const { return ""; }
 unsigned OggFile::getDuration() const { return 0; }
-FrameList* OggFile::getFrameList() const { return 0; }
-void OggFile::staticCleanup() {}
 #endif // HAVE_VORBIS
 
 /**
@@ -638,4 +856,32 @@ bool OggFile::CommentList::setValue(const QString& name, const QString& value)
 	}
 }
 
+
+/**
+ * Create an OggFile object if it supports the filename's extension.
+ *
+ * @param di directory information
+ * @param fn filename
+ *
+ * @return tagged file, 0 if type not supported.
+ */
+TaggedFile* OggFile::Resolver::createFile(const DirInfo* di,
+																					const QString& fn) const
+{
+	if (fn.right(4).QCM_toLower() == ".ogg")
+		return new OggFile(di, fn);
+	else
+		return 0;
+}
+
+/**
+ * Get a list with all extensions supported by OggFile.
+ *
+ * @return list of file extensions.
+ */
+QStringList OggFile::Resolver::getSupportedFileExtensions() const
+{
+	return QStringList() << ".ogg";
+}
+
 #endif // HAVE_VORBIS || define HAVE_FLAC
