@@ -5,11 +5,29 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 9 Jan 2003
+ *
+ * Copyright (C) 2003-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "standardtags.h"
 #include "genres.h"
 #include <qurl.h>
+#include "qtcompatmac.h"
 
 StandardTags::StandardTags()
 {
@@ -142,7 +160,7 @@ QString StandardTags::replaceEscapedChars(const QString& format)
 			'\n', '\t', '\r', '\\', '\a', '\b', '\f', '\v'};
 
 		for (int pos = 0; pos < static_cast<int>(fmt.length());) {
-			pos = fmt.find('\\', pos);
+			pos = fmt.QCM_indexOf('\\', pos);
 			if (pos == -1) break;
 			++pos;
 			for (int k = 0;; ++k) {
@@ -171,7 +189,9 @@ QString StandardTags::replaceEscapedChars(const QString& format)
  * @param replacements strings with replacements for codes
  * @param numCodes     number of elements in codes and replacements
  * @param flags        flags: FSF_SupportUrlEncode to support modifier u
- *                     (with code c "%uc") to URL encode
+ *                     (with code c "%uc") to URL encode,
+ *                     FSF_ReplaceSeparators to replace directory separators
+ *                     ('/', '\\', ':') in tags.
  *
  * @return string with percent codes replaced
  */
@@ -183,7 +203,7 @@ QString StandardTags::replacePercentCodes(
 	QString fmt(format);
 	if (!fmt.isEmpty()) {
 		for (int pos = 0; pos < static_cast<int>(fmt.length());) {
-			pos = fmt.find('%', pos);
+			pos = fmt.QCM_indexOf('%', pos);
 			if (pos == -1) break;
 
 			for (int k = 0;; ++k) {
@@ -196,15 +216,21 @@ QString StandardTags::replacePercentCodes(
 						fmt[pos + 1] == 'u' && fmt[pos + 2] == codes[k]) {
 					// code found, replace it URL encoded
 					QString enc = replacements[k];
-					QUrl::encode(enc);
+					QCM_QUrl_encode(enc);
 					fmt.replace(pos, 3, enc);
 					pos += enc.length();
 					break;
 				}
 				if (fmt[pos + 1] == codes[k]) {
 					// code found, replace it
-					fmt.replace(pos, 2, replacements[k]);
-					pos += replacements[k].length();
+					QString repl = replacements[k];
+					if (flags & FSF_ReplaceSeparators) {
+						repl.replace('/', '-');
+						repl.replace('\\', '-');
+						repl.replace(':', '-');
+					}
+					fmt.replace(pos, 2, repl);
+					pos += repl.length();
 					break;
 				}
 			}
@@ -227,7 +253,9 @@ QString StandardTags::replacePercentCodes(
  *
  * @param format format specification
  * @param flags  flags: FSF_SupportUrlEncode to support modifier u
- *               (with code c "%uc") to URL encode
+ *               (with code c "%uc") to URL encode,
+ *               FSF_ReplaceSeparators to replace directory separators
+ *               ('/', '\\', ':') in tags.
  *
  * @return formatted string.
  */
