@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 14 Apr 2007
+ *
+ * Copyright (C) 2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef STRINGLISTEDIT_H
@@ -13,7 +30,11 @@
 #include <qwidget.h>
 #include "qtcompatmac.h"
 
-class Q3ListBox;
+#if QT_VERSION >= 0x040000
+class QListWidget;
+#else
+class QListBox;
+#endif
 class QPushButton;
 
 class StringListEdit : public QWidget {
@@ -24,9 +45,8 @@ public:
 	 * Constructor.
 	 *
 	 * @param parent parent widget
-	 * @param name   Qt object name
 	 */
-	StringListEdit(QWidget* parent = 0, const char* name = 0);
+	StringListEdit(QWidget* parent = 0);
 
 	/**
 	 * Destructor.
@@ -79,7 +99,11 @@ public slots:
 	void setButtonEnableState();
 
 private:
-	Q3ListBox* m_stringListBox;
+#if QT_VERSION >= 0x040000
+	QListWidget* m_stringListBox;
+#else
+	QListBox* m_stringListBox;
+#endif
 	QPushButton* m_addPushButton;
 	QPushButton* m_moveUpPushButton;
 	QPushButton* m_moveDownPushButton;
