@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 25 Sep 2005
+ *
+ * Copyright (C) 2005-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <qdir.h>
@@ -18,6 +35,12 @@
 #include "taggedfile.h"
 #include <sys/stat.h>
 
+#if QT_VERSION >= 0x040000
+QList<const TaggedFile::Resolver*> TaggedFile::s_resolvers;
+#else
+QValueList<const TaggedFile::Resolver*> TaggedFile::s_resolvers;
+#endif
+
 /**
  * Constructor.
  *
@@ -132,12 +155,19 @@ QString TaggedFile::getGenreV1()
 }
 
 /**
- * Remove all ID3v1 tags.
+ * Remove ID3v1 frames.
  *
- * @param flt filter specifying which fields to remove
+ * @param flt filter specifying which frames to remove
  */
-void TaggedFile::removeTagsV1(const StandardTagsFilter&)
+void TaggedFile::deleteFramesV1(const FrameFilter& flt)
 {
+	if (flt.isEnabled(Frame::FT_Title))   setTitleV1("");
+	if (flt.isEnabled(Frame::FT_Artist))  setArtistV1("");
+	if (flt.isEnabled(Frame::FT_Album))   setAlbumV1("");
+	if (flt.isEnabled(Frame::FT_Comment)) setCommentV1("");
+	if (flt.isEnabled(Frame::FT_Date))    setYearV1(0);
+	if (flt.isEnabled(Frame::FT_Track))   setTrackNumV1(0);
+	if (flt.isEnabled(Frame::FT_Genre))   setGenreV1("");
 }
 
 /**
@@ -232,7 +262,7 @@ bool TaggedFile::isTagV1Supported() const
 QString TaggedFile::getAbsFilename() const
 {
 	QDir dir(m_dirInfo->getDirname());
-	return QDir::cleanDirPath(dir.absFilePath(m_newFilename));
+	return QDir::QCM_cleanPath(dir.QCM_absoluteFilePath(m_newFilename));
 }
 
 /**
@@ -344,7 +374,7 @@ void TaggedFile::setStandardTagsV2(const StandardTags* st,
  */
 static void remove_artist(QString& album)
 {
-	int pos = album.find(" - ");
+	int pos = album.QCM_indexOf(" - ");
 	if (pos != -1) {
 		album.remove(0, pos + 3);
 	}
@@ -416,7 +446,7 @@ void TaggedFile::getTagsFromFilename(StandardTags* st, QString fmt)
 	}
 	int pos = 0;
 	for (int j = 0; j < numTagCodes; ++j) {
-		pos = pattern.find('%', pos);
+		pos = pattern.QCM_indexOf('%', pos);
 		if (pos == -1) break;
 		++pos;
 		for (int k = 0; k < numTagCodes; ++k) {
@@ -438,7 +468,7 @@ void TaggedFile::getTagsFromFilename(StandardTags* st, QString fmt)
 	}
 
 	re.setPattern(pattern);
-	if (re.search(fileName) != -1) {
+	if (re.QCM_indexIn(fileName) != -1) {
 		if (idxOfCode[0])
 			st->title = re.cap(idxOfCode[0]);
 		if (idxOfCode[1])
@@ -456,7 +486,7 @@ void TaggedFile::getTagsFromFilename(StandardTags* st, QString fmt)
 
 	// album/track - artist - song
 	re.setPattern("([^/]+)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)[_ ]-[_ ]([^-_\\./ ][^/]+)\\..{3,4}$");
-	if (re.search(fn) != -1) {
+	if (re.QCM_indexIn(fn) != -1) {
 		st->album = re.cap(1);
 		st->track = re.cap(2).toInt();
 		st->artist = re.cap(3);
@@ -466,7 +496,7 @@ void TaggedFile::getTagsFromFilename(StandardTags* st, QString fmt)
 	}
 	// artist - album/track song
 	re.setPattern("([^/]+)[_ ]-[_ ]([^/]+)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{3,4}$");
-	if (re.search(fn) != -1) {
+	if (re.QCM_indexIn(fn) != -1) {
 		st->artist = re.cap(1);
 		st->album = re.cap(2);
 		st->track = re.cap(3).toInt();
@@ -475,7 +505,7 @@ void TaggedFile::getTagsFromFilename(StandardTags* st, QString fmt)
 	}
 	// /artist - album - track - song
 	re.setPattern("/([^/]+[^-_/ ])[_ ]-[_ ]([^-_/ ][^/]+[^-_/ ])[-_\\. ]+(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{3,4}$");
-	if (re.search(fn) != -1) {
+	if (re.QCM_indexIn(fn) != -1) {
 		st->artist = re.cap(1);
 		st->album = re.cap(2);
 		st->track = re.cap(3).toInt();
@@ -484,7 +514,7 @@ void TaggedFile::getTagsFromFilename(StandardTags* st, QString fmt)
 	}
 	// album/artist - track - song
 	re.setPattern("([^/]+)/([^/]+[^-_\\./ ])[-_\\. ]+(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{3,4}$");
-	if (re.search(fn) != -1) {
+	if (re.QCM_indexIn(fn) != -1) {
 		st->album = re.cap(1);
 		st->artist = re.cap(2);
 		st->track = re.cap(3).toInt();
@@ -494,7 +524,7 @@ void TaggedFile::getTagsFromFilename(StandardTags* st, QString fmt)
 	}
 	// artist/album/track song
 	re.setPattern("([^/]+)/([^/]+)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{3,4}$");
-	if (re.search(fn) != -1) {
+	if (re.QCM_indexIn(fn) != -1) {
 		st->artist = re.cap(1);
 		st->album = re.cap(2);
 		st->track = re.cap(3).toInt();
@@ -503,7 +533,7 @@ void TaggedFile::getTagsFromFilename(StandardTags* st, QString fmt)
 	}
 	// album/artist - song
 	re.setPattern("([^/]+)/([^/]+[^-_/ ])[_ ]-[_ ]([^-_/ ][^/]+)\\..{3,4}$");
-	if (re.search(fn) != -1) {
+	if (re.QCM_indexIn(fn) != -1) {
 		st->album = re.cap(1);
 		st->artist = re.cap(2);
 		st->title = re.cap(3);
@@ -533,7 +563,7 @@ QString TaggedFile::formatWithTags(const StandardTags* st, QString fmt,
 {
 	if (!isDirname) {
 		// first remove directory part from fmt
-		const int sepPos = fmt.findRev('/');
+		const int sepPos = fmt.QCM_lastIndexOf('/');
 		if (sepPos >= 0) {
 			fmt.remove(0, sepPos + 1);
 		}
@@ -541,7 +571,8 @@ QString TaggedFile::formatWithTags(const StandardTags* st, QString fmt,
 		fmt += getFileExtension();
 	}
 
-	return st->formatString(fmt);
+	return st->formatString(fmt, isDirname ?
+													StandardTags::FSF_ReplaceSeparators : 0);
 }
 
 /**
@@ -598,7 +629,7 @@ QString TaggedFile::formatTime(unsigned seconds)
 bool TaggedFile::renameFile(const QString& fnOld, const QString& fnNew) const
 {
 	QString dirname = m_dirInfo->getDirname();
-	if (fnNew.lower() == fnOld.lower()) {
+	if (fnNew.QCM_toLower() == fnOld.QCM_toLower()) {
 		// If the filenames only differ in case, the new file is reported to
 		// already exist on case insensitive filesystems (e.g. Windows),
 		// so it is checked if the new file is really the old file by
@@ -606,12 +637,12 @@ bool TaggedFile::renameFile(const QString& fnOld, const QString& fnNew) const
 		// another file would be overwritten and an error is reported.
 		if (QFile::exists(dirname + QDir::separator() + fnNew)) {
 			struct stat statOld, statNew;
-			if (::stat((dirname + QDir::separator() + fnOld).latin1(), &statOld) == 0 &&
-					::stat((dirname + QDir::separator() + fnNew).latin1(), &statNew) == 0 &&
+			if (::stat((dirname + QDir::separator() + fnOld).QCM_latin1(), &statOld) == 0 &&
+					::stat((dirname + QDir::separator() + fnNew).QCM_latin1(), &statNew) == 0 &&
 					!(statOld.st_ino == statNew.st_ino &&
 						statOld.st_dev == statNew.st_dev)) {
-				qDebug("rename(%s, %s): %s already exists", fnOld.latin1(),
-							 fnNew.latin1(), fnNew.latin1());
+				qDebug("rename(%s, %s): %s already exists", fnOld.QCM_latin1(),
+							 fnNew.QCM_latin1(), fnNew.QCM_latin1());
 				return false;
 			}
 		}
@@ -622,22 +653,22 @@ bool TaggedFile::renameFile(const QString& fnOld, const QString& fnNew) const
 		QString temp_filename(fnNew);
 		temp_filename.append("_CASE");
 		if (!QDir(dirname).rename(fnOld, temp_filename)) {
-			qDebug("rename(%s, %s) failed", fnOld.latin1(),
-					   temp_filename.latin1());
+			qDebug("rename(%s, %s) failed", fnOld.QCM_latin1(),
+					   temp_filename.QCM_latin1());
 			return false;
 		}
 		if (!QDir(dirname).rename(temp_filename, fnNew)) {
-			qDebug("rename(%s, %s) failed", temp_filename.latin1(),
-					   fnNew.latin1());
+			qDebug("rename(%s, %s) failed", temp_filename.QCM_latin1(),
+					   fnNew.QCM_latin1());
 			return false;
 		}
 	} else if (QFile::exists(dirname + QDir::separator() + fnNew)) {
-		qDebug("rename(%s, %s): %s already exists", fnOld.latin1(),
-					 fnNew.latin1(), fnNew.latin1());
+		qDebug("rename(%s, %s): %s already exists", fnOld.QCM_latin1(),
+					 fnNew.QCM_latin1(), fnNew.QCM_latin1());
 		return false;
 	} else if (!QDir(dirname).rename(fnOld, fnNew)) {
-		qDebug("rename(%s, %s) failed", fnOld.latin1(),
-					 fnNew.latin1());
+		qDebug("rename(%s, %s) failed", fnOld.QCM_latin1(),
+					 fnNew.QCM_latin1());
 		return false;
 	}
 	return true;
@@ -669,35 +700,38 @@ int TaggedFile::getTotalNumberOfTracksIfEnabled() const
 }
 
 /**
- * Remove the standard ID3v1 tags.
+ * Add the total number of tracks to the track number if enabled.
  *
- * @param flt filter specifying which fields to remove
+ * @param value string containing track number,
+ *              "/t" with t = total number of tracks will be appended
+ *              if enabled and value contains a number
  */
-void TaggedFile::removeStandardTagsV1(const StandardTagsFilter& flt)
+void TaggedFile::addTotalNumberOfTracksIfEnabled(QString& value) const
 {
-	if (flt.m_enableTitle)   setTitleV1("");
-	if (flt.m_enableArtist)  setArtistV1("");
-	if (flt.m_enableAlbum)   setAlbumV1("");
-	if (flt.m_enableComment) setCommentV1("");
-	if (flt.m_enableYear)    setYearV1(0);
-	if (flt.m_enableTrack)   setTrackNumV1(0);
-	if (flt.m_enableGenre)   setGenreV1("");
+	int numTracks = getTotalNumberOfTracksIfEnabled();
+	if (numTracks > 0) {
+		bool ok;
+		int trackNr = value.toInt(&ok);
+		if (ok && trackNr > 0) {
+			value = QString("%1/%2").arg(trackNr).arg(numTracks);
+		}
+	}
 }
 
 /**
- * Remove the standard ID3v2 tags.
+ * Remove ID3v2 frames.
  *
- * @param flt filter specifying which fields to remove
+ * @param flt filter specifying which frames to remove
  */
-void TaggedFile::removeStandardTagsV2(const StandardTagsFilter& flt)
+void TaggedFile::deleteFramesV2(const FrameFilter& flt)
 {
-	if (flt.m_enableTitle)   setTitleV2("");
-	if (flt.m_enableArtist)  setArtistV2("");
-	if (flt.m_enableAlbum)   setAlbumV2("");
-	if (flt.m_enableComment) setCommentV2("");
-	if (flt.m_enableYear)    setYearV2(0);
-	if (flt.m_enableTrack)   setTrackNumV2(0);
-	if (flt.m_enableGenre)   setGenreV2("");
+	if (flt.isEnabled(Frame::FT_Title))   setTitleV2("");
+	if (flt.isEnabled(Frame::FT_Artist))  setArtistV2("");
+	if (flt.isEnabled(Frame::FT_Album))   setAlbumV2("");
+	if (flt.isEnabled(Frame::FT_Comment)) setCommentV2("");
+	if (flt.isEnabled(Frame::FT_Date))    setYearV2(0);
+	if (flt.isEnabled(Frame::FT_Track))   setTrackNumV2(0);
+	if (flt.isEnabled(Frame::FT_Genre))   setGenreV2("");
 }
 
 /**
@@ -767,3 +801,439 @@ int TaggedFile::checkTruncation(int val, StandardTags::TruncationFlag flag,
 		return -1;
 	}
 }
+
+/**
+ * Get a specific frame from the tags 1.
+ *
+ * @param type  frame type
+ * @param frame the frame is returned here
+ *
+ * @return true if ok.
+ */
+bool TaggedFile::getFrameV1(Frame::Type type, Frame& frame)
+{
+	int n = -1;
+	bool number = false;
+
+	switch (type) {
+		case Frame::FT_Album:
+			frame.m_value = getAlbumV1();
+			break;
+		case Frame::FT_Artist:
+			frame.m_value = getArtistV1();
+			break;
+		case Frame::FT_Comment:
+			frame.m_value = getCommentV1();
+			break;
+		case Frame::FT_Date:
+			n = getYearV1();
+			number = true;
+			break;
+		case Frame::FT_Genre:
+			frame.m_value = getGenreV1();
+			break;
+		case Frame::FT_Title:
+			frame.m_value = getTitleV1();
+			break;
+		case Frame::FT_Track:
+			n = getTrackNumV1();
+			number = true;
+			break;
+		default:
+			// maybe handled in a subclass
+			return false;
+	}
+	if (number) {
+		if (n == -1) {
+			frame.m_value = QString();
+		} else if (n == 0) {
+			frame.m_value = QString("");
+		} else {
+			frame.m_value.setNum(n);
+		}
+	}
+	frame.m_type = type;
+	return true;
+}
+
+/**
+ * Set a frame in the tags 1.
+ *
+ * @param frame frame to set.
+ *
+ * @return true if ok.
+ */
+bool TaggedFile::setFrameV1(const Frame& frame)
+{
+	int n = -1;
+	if (frame.m_type == Frame::FT_Date ||
+			frame.m_type == Frame::FT_Track) {
+		if (frame.isInactive()) {
+			n = -1;
+		} else if (frame.isEmpty()) {
+			n = 0;
+		} else {
+			n = frame.m_value.toInt();
+		} 
+	}
+	switch (frame.m_type) {
+		case Frame::FT_Album:
+			setAlbumV1(frame.m_value);
+			break;
+		case Frame::FT_Artist:
+			setArtistV1(frame.m_value);
+			break;
+		case Frame::FT_Comment:
+			setCommentV1(frame.m_value);
+			break;
+		case Frame::FT_Date:
+			setYearV1(n);
+			break;
+		case Frame::FT_Genre:
+			setGenreV1(frame.m_value);
+			break;
+		case Frame::FT_Title:
+			setTitleV1(frame.m_value);
+			break;
+		case Frame::FT_Track:
+			setTrackNumV1(n);
+			break;
+		default:
+			// maybe handled in a subclass
+			return false;
+	}
+	return true;
+}
+
+/**
+ * Get a specific frame from the tags 2.
+ *
+ * @param type  frame type
+ * @param frame the frame is returned here
+ *
+ * @return true if ok.
+ */
+bool TaggedFile::getFrameV2(Frame::Type type, Frame& frame)
+{
+	int n = -1;
+	bool number = false;
+
+	switch (type) {
+		case Frame::FT_Album:
+			frame.m_value = getAlbumV2();
+			break;
+		case Frame::FT_Artist:
+			frame.m_value = getArtistV2();
+			break;
+		case Frame::FT_Comment:
+			frame.m_value = getCommentV2();
+			break;
+		case Frame::FT_Date:
+			n = getYearV2();
+			number = true;
+			break;
+		case Frame::FT_Genre:
+			frame.m_value = getGenreV2();
+			break;
+		case Frame::FT_Title:
+			frame.m_value = getTitleV2();
+			break;
+		case Frame::FT_Track:
+			n = getTrackNumV2();
+			number = true;
+			break;
+		default:
+			// maybe handled in a subclass
+			return false;
+	}
+	if (number) {
+		if (n == -1) {
+			frame.m_value = QString();
+		} else if (n == 0) {
+			frame.m_value = QString("");
+		} else {
+			frame.m_value.setNum(n);
+		}
+	}
+	frame.m_type = type;
+	return true;
+}
+
+/**
+ * Set a frame in the tags 2.
+ *
+ * @param frame frame to set
+ *
+ * @return true if ok.
+ */
+bool TaggedFile::setFrameV2(const Frame& frame)
+{
+	int n = -1;
+	if (frame.m_type == Frame::FT_Date ||
+			frame.m_type == Frame::FT_Track) {
+		if (frame.isInactive()) {
+			n = -1;
+		} else if (frame.isEmpty()) {
+			n = 0;
+		} else {
+			n = frame.m_value.toInt();
+		} 
+	}
+	switch (frame.m_type) {
+		case Frame::FT_Album:
+			setAlbumV2(frame.m_value);
+			break;
+		case Frame::FT_Artist:
+			setArtistV2(frame.m_value);
+			break;
+		case Frame::FT_Comment:
+			setCommentV2(frame.m_value);
+			break;
+		case Frame::FT_Date:
+			setYearV2(n);
+			break;
+		case Frame::FT_Genre:
+			setGenreV2(frame.m_value);
+			break;
+		case Frame::FT_Title:
+			setTitleV2(frame.m_value);
+			break;
+		case Frame::FT_Track:
+			setTrackNumV2(n);
+			break;
+		default:
+			// maybe handled in a subclass
+			return false;
+	}
+	return true;
+}
+
+/**
+ * Add a frame in the tags 2.
+ *
+ * @param frame frame to add, a field list may be added by this method
+ *
+ * @return true if ok.
+ */
+bool TaggedFile::addFrameV2(Frame& frame)
+{
+	return TaggedFile::setFrameV2(frame);
+}
+
+/**
+ * Delete a frame in the tags 2.
+ *
+ * @param frame frame to delete.
+ *
+ * @return true if ok.
+ */
+bool TaggedFile::deleteFrameV2(const Frame& frame)
+{
+	Frame emptyFrame(frame);
+	emptyFrame.setValue("");
+	return setFrameV2(emptyFrame);
+}
+
+/**
+ * Get all frames in tag 1.
+ *
+ * @param frames frame collection to set.
+ */
+void TaggedFile::getAllFramesV1(FrameCollection& frames)
+{
+	frames.clear();
+	Frame frame;
+	for (int i = Frame::FT_FirstFrame; i <= Frame::FT_LastV1Frame; ++i) {
+		if (getFrameV1(static_cast<Frame::Type>(i), frame)) {
+			frames.insert(frame);
+		}
+	}
+}
+
+/**
+ * Set frames in tag 1.
+ *
+ * @param frames      frame collection
+ * @param onlyChanged only frames with value marked as changed are set
+ */
+void TaggedFile::setFramesV1(const FrameCollection& frames, bool onlyChanged)
+{
+	for (FrameCollection::const_iterator it = frames.begin();
+			 it != frames.end();
+			 ++it) {
+		if (!onlyChanged || it->isValueChanged()) {
+				setFrameV1(*it);
+				markTag1Changed();
+		}
+	}
+}
+
+/**
+ * Get all frames in tag 2.
+ * This generic implementation only supports the standard tags and should
+ * be reimplemented in derived classes.
+ *
+ * @param frames frame collection to set.
+ */
+void TaggedFile::getAllFramesV2(FrameCollection& frames)
+{
+	frames.clear();
+	Frame frame;
+	for (int i = Frame::FT_FirstFrame; i <= Frame::FT_LastV1Frame; ++i) {
+		if (getFrameV2(static_cast<Frame::Type>(i), frame)) {
+			frames.insert(frame);
+		}
+	}
+}
+
+/**
+ * Set frames in tag 2.
+ *
+ * @param frames      frame collection
+ * @param onlyChanged only frames with value marked as changed are set
+ */
+void TaggedFile::setFramesV2(const FrameCollection& frames, bool onlyChanged)
+{
+	bool myFramesValid = false;
+	FrameCollection myFrames;
+
+	for (FrameCollection::const_iterator it = frames.begin();
+			 it != frames.end();
+			 ++it) {
+		if (!onlyChanged || it->isValueChanged()) {
+			if (it->getIndex() != -1) {
+				// The frame has an index, so the original tag can be modified
+				setFrameV2(*it);
+				markTag2Changed();
+			} else {
+				// The frame does not have an index
+				if (it->getType() <= Frame::FT_LastV1Frame) {
+					// Standard tags can be handled with the basic method
+					TaggedFile::setFrameV2(*it);
+					markTag2Changed();
+				} else {
+					// The frame has to be looked up and modified
+					if (!myFramesValid) {
+						getAllFramesV2(myFrames);
+						myFramesValid = true;
+					}
+					FrameCollection::iterator myIt = myFrames.find(*it);
+					if (myIt != myFrames.end()) {
+						Frame myFrame(*it);
+						myFrame.setIndex(myIt->getIndex());
+						setFrameV2(myFrame);
+						markTag2Changed();
+					} else {
+						// Such a frame does not exist, add a new one.
+						Frame myFrame(*it);
+						addFrameV2(myFrame);
+						setFrameV2(myFrame);
+						markTag2Changed();
+					}
+				}
+			}
+		}
+	}
+}
+
+/**
+ * Get a list of frame IDs which can be added.
+ *
+ * @return list with frame IDs.
+ */
+QStringList TaggedFile::getFrameIds() const
+{
+	QStringList lst;
+	for (int i = Frame::FT_FirstFrame; i <= Frame::FT_LastFrame; ++i) {
+		lst.append(QCM_translate(Frame::getNameFromType(static_cast<Frame::Type>(i))));
+	}
+	return lst;
+}
+
+
+/**
+ * Add a file type resolver to the end of a list of resolvers.
+ *
+ * @param resolver file type resolver to add
+ */
+void TaggedFile::addResolver(const Resolver* resolver)
+{
+	s_resolvers.push_back(resolver);
+}
+
+/**
+ * Create a TaggedFile subclass using the first successful resolver.
+ * @see addResolver()
+ *
+ * @param di directory information
+ * @param fn filename
+ *
+ * @return tagged file, 0 if type not supported.
+ */
+TaggedFile* TaggedFile::createFile(const DirInfo* di, const QString& fn)
+{
+	TaggedFile* taggedFile = 0;
+	for (
+#if QT_VERSION >= 0x040000
+		QList<const Resolver*>::const_iterator
+#else
+		QValueList<const Resolver*>::const_iterator
+#endif
+			it = s_resolvers.begin(); it != s_resolvers.end(); ++it) {
+		taggedFile = (*it)->createFile(di, fn);
+		if (taggedFile) break;
+	}
+	return taggedFile;
+}
+
+/**
+ * Get a list with all extensions (e.g. ".mp3") supported by the resolvers.
+ * @see addResolver()
+ *
+ * @return list of file extensions.
+ */
+QStringList TaggedFile::getSupportedFileExtensions()
+{
+	QStringList extensions;
+	for (
+#if QT_VERSION >= 0x040000
+		QList<const Resolver*>::const_iterator
+#else
+		QValueList<const Resolver*>::const_iterator
+#endif
+			it = s_resolvers.begin(); it != s_resolvers.end(); ++it) {
+		extensions += (*it)->getSupportedFileExtensions();
+	}
+
+	// remove duplicates
+	extensions.sort();
+	QString lastExt("");
+	for (QStringList::iterator it = extensions.begin();
+			 it != extensions.end();) {
+		if (*it == lastExt) {
+			it = extensions.erase(it);
+		} else {
+			lastExt = *it;
+			++it;
+		}
+	}
+
+	return extensions;
+}
+
+/**
+ * Free static resources.
+ */
+void TaggedFile::staticCleanup()
+{
+#if QT_VERSION >= 0x040000
+	qDeleteAll(s_resolvers);
+#else
+	for (QValueList<const Resolver*>::const_iterator it = s_resolvers.begin();
+			 it != s_resolvers.end();
+			 ++it) {
+		delete *it;
+	}
+#endif
+	s_resolvers.clear();
+}
