@@ -5,20 +5,76 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 25 Sep 2005
+ *
+ * Copyright (C) 2005-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef TAGGEDFILE_H
 #define TAGGEDFILE_H
 
 #include <qstring.h>
+#include <qstringlist.h>
+#include "qtcompatmac.h"
+#if QT_VERSION >= 0x040000
+#include <QList>
+#else
+#include <qvaluelist.h>
+#endif
 #include "standardtags.h"
+#include "frame.h"
 
-class FrameList;
 class DirInfo;
 
 /** Base class for tagged files. */
 class TaggedFile {
 public:
+	/** Abstract base class for pluggable file type resolution. */
+	class Resolver {
+	public:
+		/**
+		 * Constructor.
+		 */
+		Resolver() {}
+
+		/**
+		 * Destructor.
+		 */
+		virtual ~Resolver() {}
+
+		/**
+		 * Create a TaggedFile subclass depending on the file extension.
+		 *
+		 * @param di directory information
+		 * @param fn filename
+		 *
+		 * @return tagged file, 0 if type not supported.
+		 */
+		virtual TaggedFile* createFile(const DirInfo* di, const QString& fn) const = 0;
+
+		/**
+		 * Get a list with all extensions (e.g. ".mp3") supported by TaggedFile subclass.
+		 *
+		 * @return list of file extensions.
+		 */
+		virtual QStringList getSupportedFileExtensions() const = 0;
+	};
+
+
 	/**
 	 * Constructor.
 	 *
@@ -144,11 +200,11 @@ public:
 	virtual QString getGenreV1();
 
 	/**
-	 * Remove all ID3v1 tags.
+	 * Remove ID3v1 frames.
 	 *
-	 * @param flt filter specifying which fields to remove
+	 * @param flt filter specifying which frames to remove
 	 */
-	virtual void removeTagsV1(const StandardTagsFilter& flt);
+	virtual void deleteFramesV1(const FrameFilter& flt);
 
 	/**
 	 * Set ID3v1 title.
@@ -269,11 +325,11 @@ public:
 	virtual int getTrackNumV2() = 0;
 
 	/**
-	 * Remove all ID3v2 tags.
+	 * Remove ID3v2 frames.
 	 *
-	 * @param flt filter specifying which fields to remove
+	 * @param flt filter specifying which frames to remove
 	 */
-	virtual void removeTagsV2(const StandardTagsFilter& flt) = 0;
+	virtual void deleteFramesV2(const FrameFilter& flt);
 
 	/**
 	 * Get ID3v2 genre as text.
@@ -366,13 +422,6 @@ public:
 	 */
 	virtual unsigned getDuration() const = 0;
 
-	/**
-	 * Get frame list for this type of tagged file.
-	 *
-	 * @return frame list.
-	 */
-	virtual FrameList* getFrameList() const = 0;
-
 	/**
 	 * Get file extension including the dot.
 	 *
@@ -398,6 +447,101 @@ public:
 	 */
 	virtual QString getTagFormatV2() const;
 
+	/**
+	 * Get a specific frame from the tags 1.
+	 *
+	 * @param type  frame type
+	 * @param frame the frame is returned here
+	 *
+	 * @return true if ok.
+	 */
+	virtual bool getFrameV1(Frame::Type type, Frame& frame);
+
+	/**
+	 * Set a frame in the tags 1.
+	 *
+	 * @param frame frame to set.
+	 *
+	 * @return true if ok.
+	 */
+	virtual bool setFrameV1(const Frame& frame);
+
+	/**
+	 * Get a specific frame from the tags 2.
+	 *
+	 * @param type  frame type
+	 * @param frame the frame is returned here
+	 *
+	 * @return true if ok.
+	 */
+	virtual bool getFrameV2(Frame::Type type, Frame& frame);
+
+	/**
+	 * Set a frame in the tags 2.
+	 *
+	 * @param frame frame to set
+	 *
+	 * @return true if ok.
+	 */
+	virtual bool setFrameV2(const Frame& frame);
+
+	/**
+	 * Add a frame in the tags 2.
+	 *
+	 * @param frame frame to add, a field list may be added by this method
+	 *
+	 * @return true if ok.
+	 */
+	virtual bool addFrameV2(Frame& frame);
+
+	/**
+	 * Delete a frame in the tags 2.
+	 *
+	 * @param frame frame to delete.
+	 *
+	 * @return true if ok.
+	 */
+	virtual bool deleteFrameV2(const Frame& frame);
+
+	/**
+	 * Get a list of frame IDs which can be added.
+	 *
+	 * @return list with frame IDs.
+	 */
+	virtual QStringList getFrameIds() const;
+
+	/**
+	 * Get all frames in tag 1.
+	 *
+	 * @param frames frame collection to set.
+	 */
+	virtual void getAllFramesV1(FrameCollection& frames);
+
+	/**
+	 * Get all frames in tag 2.
+	 * This generic implementation only supports the standard tags and should
+	 * be reimplemented in derived classes.
+	 *
+	 * @param frames frame collection to set.
+	 */
+	virtual void getAllFramesV2(FrameCollection& frames);
+
+	/**
+	 * Set frames in tag 1.
+	 *
+	 * @param frames      frame collection
+	 * @param onlyChanged only frames with value marked as changed are set
+	 */
+	void setFramesV1(const FrameCollection& frames, bool onlyChanged = true);
+
+	/**
+	 * Set frames in tag 2.
+	 *
+	 * @param frames      frame collection
+	 * @param onlyChanged only frames with value marked as changed are set
+	 */
+	void setFramesV2(const FrameCollection& frames, bool onlyChanged = true);
+
 	/**
 	 * Get ID3v1 tags from file.
 	 *
@@ -511,6 +655,15 @@ public:
 	 */
 	unsigned getTruncationFlags() const { return m_truncation; }
 
+	/**
+	 * Add the total number of tracks to the track number if enabled.
+	 *
+	 * @param value string containing track number,
+	 *              "/t" with t = total number of tracks will be appended
+	 *              if enabled and value contains a number
+	 */
+	void addTotalNumberOfTracksIfEnabled(QString& value) const;
+
 	/**
 	 * Format a time string "h:mm:ss".
 	 * If the time is less than an hour, the hour is not put into the
@@ -522,6 +675,37 @@ public:
 	 */
 	static QString formatTime(unsigned seconds);
 
+	/**
+	 * Add a file type resolver to the end of a list of resolvers.
+	 *
+	 * @param resolver file type resolver to add
+	 */
+	static void addResolver(const Resolver* resolver);
+
+	/**
+	 * Create a TaggedFile subclass using the first successful resolver.
+	 * @see addResolver()
+	 *
+	 * @param di directory information
+	 * @param fn filename
+	 *
+	 * @return tagged file, 0 if type not supported.
+	 */
+	static TaggedFile* createFile(const DirInfo* di, const QString& fn);
+
+	/**
+	 * Get a list with all extensions (e.g. ".mp3") supported by the resolvers.
+	 * @see addResolver()
+	 *
+	 * @return list of file extensions.
+	 */
+	static QStringList getSupportedFileExtensions();
+
+	/**
+	 * Free static resources.
+	 */
+	static void staticCleanup();
+
 protected:
 	/**
 	 * Rename a file.
@@ -549,20 +733,6 @@ protected:
 	*/
 	int getTotalNumberOfTracksIfEnabled() const;
 
-	/**
-	 * Remove the standard ID3v1 tags.
-	 *
-	 * @param flt filter specifying which fields to remove
-	 */
-	void removeStandardTagsV1(const StandardTagsFilter& flt);
-
-	/**
-	 * Remove the standard ID3v2 tags.
-	 *
-	 * @param flt filter specifying which fields to remove
-	 */
-	void removeStandardTagsV2(const StandardTagsFilter& flt);
-
 	/**
 	 * Get current filename.
 	 * @return existing name.
@@ -634,6 +804,12 @@ private:
 	bool m_changedV2;
 	/** Truncation flags. */
 	unsigned m_truncation;
+
+#if QT_VERSION >= 0x040000
+	static QList<const Resolver*> s_resolvers;
+#else
+	static QValueList<const Resolver*> s_resolvers;
+#endif
 };
 
 #endif // TAGGEDFILE_H
