@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 12 Sep 2006
+ *
+ * Copyright (C) 2006-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef TAGLIBFILE_H
@@ -14,8 +31,11 @@
 #ifdef HAVE_TAGLIB
 
 #include "taggedfile.h"
+#include "miscconfig.h"
 #include <taglib/fileref.h>
 #include <taglib/tag.h>
+#include <taglib/id3v2frame.h>
+#include <qglobal.h>
 
 /** TagLib version in with 8 bits for major, minor and patch version. */
 #define TAGLIB_VERSION (((TAGLIB_MAJOR_VERSION) << 16) + \
@@ -30,11 +50,37 @@
 //#define TAGLIB_SUPPORTS_USLT_FRAMES
 #endif
 
-class TagLibFrameList;
+#if QT_VERSION >= 0x040000
+#define QSTRING_TO_TSTRING(s) TagLib::String(s.toUtf8().data(), TagLib::String::UTF8)
+#else
+#define QSTRING_TO_TSTRING(s) TagLib::String(s.utf8().data(), TagLib::String::UTF8)
+#endif
+
 
 /** List box item containing tagged file. */
 class TagLibFile : public TaggedFile {
 public:
+	/** File type resolution. */
+	class Resolver : public TaggedFile::Resolver {
+		/**
+		 * Create an TagLibFile object if it supports the filename's extension.
+		 *
+		 * @param di directory information
+		 * @param fn filename
+		 *
+		 * @return tagged file, 0 if type not supported.
+		 */
+		virtual TaggedFile* createFile(const DirInfo* di, const QString& fn) const;
+
+		/**
+		 * Get a list with all extensions supported by TagLibFile.
+		 *
+		 * @return list of file extensions.
+		 */
+		virtual QStringList getSupportedFileExtensions() const;
+	};
+
+
 	/**
 	 * Constructor.
 	 *
@@ -69,18 +115,18 @@ public:
 	virtual bool writeTags(bool force, bool* renamed, bool preserve);
 
 	/**
-	 * Remove all ID3v1 tags.
+	 * Remove ID3v1 frames.
 	 *
-	 * @param flt filter specifying which fields to remove
+	 * @param flt filter specifying which frames to remove
 	 */
-	virtual void removeTagsV1(const StandardTagsFilter& flt);
+	virtual void deleteFramesV1(const FrameFilter& flt);
 
 	/**
-	 * Remove all ID3v2 tags.
+	 * Remove ID3v2 frames.
 	 *
-	 * @param flt filter specifying which fields to remove
+	 * @param flt filter specifying which frames to remove
 	 */
-	virtual void removeTagsV2(const StandardTagsFilter& flt);
+	virtual void deleteFramesV2(const FrameFilter& flt);
 
 	/**
 	 * Get ID3v1 title.
@@ -354,13 +400,6 @@ public:
 	 */
 	virtual unsigned getDuration() const;
 
-	/**
-	 * Get frame list for this type of tagged file.
-	 *
-	 * @return frame list.
-	 */
-	virtual FrameList* getFrameList() const;
-
 	/**
 	 * Get file extension including the dot.
 	 *
@@ -387,13 +426,75 @@ public:
 	virtual QString getTagFormatV2() const;
 
 	/**
-	 * Clean up static resources.
+	 * Set a frame in the tags 2.
+	 *
+	 * @param frame frame to set
+	 *
+	 * @return true if ok.
 	 */
-	static void staticCleanup();
+	virtual bool setFrameV2(const Frame& frame);
 
-	friend class TagLibFrameList;
+	/**
+	 * Add a frame in the tags 2.
+	 *
+	 * @param frame frame to add, a field list may be added by this method
+	 *
+	 * @return true if ok.
+	 */
+	virtual bool addFrameV2(Frame& frame);
+
+	/**
+	 * Delete a frame in the tags 2.
+	 *
+	 * @param frame frame to delete.
+	 *
+	 * @return true if ok.
+	 */
+	virtual bool deleteFrameV2(const Frame& frame);
+
+	/**
+	 * Get all frames in tag 2.
+	 *
+	 * @param frames frame collection to set.
+	 */
+	virtual void getAllFramesV2(FrameCollection& frames);
+
+	/**
+	 * Get a list of frame IDs which can be added.
+	 *
+	 * @return list with frame IDs.
+	 */
+	virtual QStringList getFrameIds() const;
+
+	/**
+	 * Static initialization.
+	 * Registers file types.
+	 */
+	static void staticInit();
+
+	/**
+	 * Set the default text encoding.
+	 *
+	 * @param textEnc default text encoding
+	 */
+	static void setDefaultTextEncoding(MiscConfig::TextEncoding textEnc);
+
+	/**
+	 * Get the default text encoding.
+	 * @return default text encoding.
+	 */
+	static TagLib::String::Type getDefaultTextEncoding() { return s_defaultTextEncoding; }
 
 private:
+	/**
+	 * Modify an ID3v2 frame.
+	 *
+	 * @param id3Frame original ID3v2 frame
+	 * @param frame    frame with fields to set in new frame
+	 */
+	void setId3v2Frame(
+		TagLib::ID3v2::Frame* id3Frame, const Frame& frame) const;
+
 	TagLibFile(const TagLibFile&);
 	TagLibFile& operator=(const TagLibFile&);
 
@@ -416,8 +517,8 @@ private:
 	TagLib::Tag* m_tagV2;      /**< ID3v2 tags */
 	bool m_fileRead;           /**< true if file has been read */
 
-	/** Frame list for MP3 files. */
-	static TagLibFrameList* s_tagLibFrameList;
+	/** default text encoding */
+	static TagLib::String::Type s_defaultTextEncoding;
 };
 
 #endif // HAVE_TAGLIB
